@@ -18,9 +18,22 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-let get_default_hostname ?sp () = Url.Current.host
-let get_default_port ?sp () = 80 (*VVV take from server? !!!!!!!!! *) (*RRR ??? Url.default_http_port ???*)
-let get_default_sslport ?sp () = 443 (*VVV take from server? !!!!!!!!! *) (*RRR ??? replace by Url.default_https_port ???*)
-let get_default_links_xhr ?sp () = true (*BBB take from server? !!!!!!!!!! *)
+let get_default_hostname () = Url.Current.host
+let get_default_port () = 80 (*VVV take from server? !!!!!!!!! *) (*RRR ??? Url.default_http_port ???*)
+let get_default_sslport () = 443 (*VVV take from server? !!!!!!!!! *) (*RRR ??? replace by Url.default_https_port ???*)
+let get_default_links_xhr () = true (*BBB take from server? !!!!!!!!!! *)
 
 let debug_timings = ref false
+
+let is_tracing =
+  ref false
+let set_tracing value =
+  is_tracing := value
+let get_tracing () =
+  !is_tracing
+
+(* let () = *)
+(*   if Js.to_string Dom_html.window##location##hash = "#__trace" then *)
+(*     set_tracing true; *)
+(*   if Js.to_string Dom_html.window##location##hash = "#__timings" then *)
+(*     debug_timings := true *)
