@@ -20,7 +20,7 @@
 
 open Eliom_lib
 
-(** This module provides the creation of valid XML content, i.e. XML, XHTML, SVG,
+(** This module provides the creation of valid XML content, i.e. XML, SVG,
     and (X)HTML5.
     See {% <<a_api subproject="server" | module Eliom_content >> %} for an explication
     of the modules [F] and [D].
@@ -42,7 +42,7 @@ module Svg : module type of Eliom_content_core.Svg
 module Html5 : sig
 
   (** See the Eliom manual for more information on {% <<a_manual
-      chapter="client" fragment="unique"| dom semantics vs. functional
+      chapter="clientserver-html" fragment="unique"| dom semantics vs. functional
       semantics>> %} for HTML5 tree manipulated by client/server
       application. *)
 
@@ -63,6 +63,31 @@ module Html5 : sig
         and type 'a attrib = 'a attrib
         and type uri = uri
     include "sigs/eliom_html5_forms.mli"
+
+    (** Creates an untyped form. *)
+    val raw_form : ([< Html5_types.form_attrib ], [< Html5_types.form_content_fun ], [> Html5_types.form ]) plus
+
+    (** This is an alias to
+        {% <<a_api|val Eliom_content.Html5.F.get_form>> %}
+        to avoid the untyped [Eliom_content_core.Html5.F.form]. *)
+    val form : ?absolute:bool -> ?absolute_path:bool -> ?https:bool -> ?a:Html5_types.form_attrib attrib list ->
+      service:('get, unit, [< get_service_kind ], [<suff ], 'gn, 'pn, [< registrable ], 'return) service ->
+      ?hostname:string -> ?port:int -> ?fragment:string -> ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameter.nl_params_set -> ?xhr:bool ->
+      ('gn -> Html5_types.form_content elt list) -> [> Html5_types.form ] elt
+
+    (** This is an alias to
+        {% <<a_api|val Eliom_content.Html5.F.string_input>> %}
+        to avoid the untyped [Eliom_content_core.Html5.F.input]. *)
+    val input : ?a:Html5_types.input_attrib attrib list -> input_type:[<
+	| `Url | `Tel | `Text | `Time | `Search | `Password | `Checkbox | `Range | `Radio | `Submit | `Reset | `Number | `Hidden
+	| `Month | `Week | `File | `Email | `Image | `Datetime_local | `Datetime | `Date | `Color | `Button]
+      -> ?name:[< string setoneradio ] param_name -> ?value:string -> unit -> [> Html5_types.input ] elt
+
+    (** This is an alias to
+        {% <<a_api|val Eliom_content.Html5.F.string_select>> %}
+        to avoid the untyped [Eliom_content_core.Html5.F.select]. *)
+    val select : ?a:Html5_types.select_attrib attrib list -> name:[< `One of string ] param_name -> string select_opt -> string select_opt list -> [> Html5_types.select ] elt
   end
 
   (** Creation of HTML5 content with {e D}OM semantics (referable) *)
@@ -78,10 +103,45 @@ module Html5 : sig
         and type 'a attrib = 'a attrib
         and type uri = uri
     include "sigs/eliom_html5_forms.mli"
+
+    (** Creates an untyped form. *)
+    val raw_form : ([< Html5_types.form_attrib ], [< Html5_types.form_content_fun ], [> Html5_types.form ]) plus
+
+    (** This is an alias to
+        {% <<a_api|val Eliom_content.Html5.D.get_form>> %}
+        to avoid the untyped [Eliom_content_core.Html5.D.form]. *)
+    val form : ?absolute:bool -> ?absolute_path:bool -> ?https:bool -> ?a:Html5_types.form_attrib attrib list ->
+      service:('get, unit, [< get_service_kind ], [<suff ], 'gn, 'pn, [< registrable ], 'return) service ->
+      ?hostname:string -> ?port:int -> ?fragment:string -> ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameter.nl_params_set -> ?xhr:bool ->
+      ('gn -> Html5_types.form_content elt list) -> [> Html5_types.form ] elt
+
+    (** This is an alias to
+        {% <<a_api|val Eliom_content.Html5.D.string_input>> %}
+        to avoid the untyped [Eliom_content_core.Html5.D.input]. *)
+    val input : ?a:Html5_types.input_attrib attrib list -> input_type:[<
+	| `Url | `Tel | `Text | `Time | `Search | `Password | `Checkbox | `Range | `Radio | `Submit | `Reset | `Number | `Hidden
+	| `Month | `Week | `File | `Email | `Image | `Datetime_local | `Datetime | `Date | `Color | `Button]
+      -> ?name:[< string setoneradio ] param_name -> ?value:string -> unit -> [> Html5_types.input ] elt
+
+    (** This is an alias to
+        {% <<a_api|val Eliom_content.Html5.D.string_select>> %}
+        to avoid the untyped [Eliom_content_core.Html5.D.select]. *)
+    val select : ?a:Html5_types.select_attrib attrib list -> name:[< `One of string ] param_name -> string select_opt -> string select_opt list -> [> Html5_types.select ] elt
   end
 
   (** Node identifiers *)
-  module Id : module type of Eliom_content_core.Html5.Id
+  module Id : sig
+    include module type of Eliom_content_core.Html5.Id
+                             with type +'a id = 'a Eliom_content_core.Html5.Id.id
+
+    (** [get_element id] returns the HTML element in the DOM with the given [id].
+        @raises Not_found if the [id] was no such element. *)
+    val get_element : 'a id -> 'a elt
+  end
+
+  module Custom_data : module type of Eliom_content_core.Html5.Custom_data
+                                        with type 'a t = 'a Eliom_content_core.Html5.Custom_data.t
 
   (** Conversion from HTML5 [elt]s to Javascript DOM elements ([<:] {% <<a_api
       project="js_of_ocaml"| class Dom_html.element >> %}).
@@ -89,6 +149,8 @@ module Html5 : sig
   module To_dom : sig
 
     val of_element : 'a elt -> Dom_html.element Js.t
+    val of_node : 'a elt -> Dom.node Js.t
+
     val of_heading : Html5_types.heading elt -> Dom_html.headingElement Js.t
 
     val of_pcdata : [> `Pcdata] elt -> Dom.text Js.t
@@ -220,7 +282,7 @@ module Html5 : sig
   (** DOM-like manipulation functions.
 
       In this module, all the functions apply only to HTML5 element with
-      {% <<a_manual chapter="client" fragment="unique"|Dom semantics>>
+      {% <<a_manual chapter="clientserver-html" fragment="unique"|Dom semantics>>
       %}.
   *)
   module Manip : sig
@@ -306,6 +368,11 @@ module Html5 : sig
     val childElements: 'a elt -> Dom.element Js.t list
     (**/**)
 
+(*
+    val get_custom_data : _ elt -> 'a Custom_data.t -> 'a
+    val set_custom_data : _ elt -> 'a Custom_data.t -> 'a -> unit
+ *)
+
     (** Read the CSS properties of DOM elements. *)
     module Css : sig
       val background: 'a elt -> string
@@ -510,4 +577,3 @@ module Html5 : sig
   module Of_dom : module type of Eliom_content_core.Html5.Of_dom
 
 end
-
