@@ -42,20 +42,28 @@ module Xml = struct
   and elt = {
     (* See Eliom_content.Html5.To_dom for the 'unwrap' function that convert
        the server's tree representation into the client one. *)
-    mutable elt : node;
+    mutable elt : node lazy_t;
     node_id : node_id;
   }
 
   let content e =
-    match e.elt with
+    match Lazy.force e.elt with
     | DomNode _ -> assert false (* TODO *)
     | TyXMLNode elt -> elt
-  let get_node e = e.elt
-  let set_dom_node elt node = elt.elt <- DomNode node
+  let get_node e = Lazy.force e.elt
+  let set_dom_node elt node = elt.elt <- Lazy.lazy_from_val (DomNode node)
   let get_node_id elt = elt.node_id
 
-  let make ?(id = NoId) elt = { elt = TyXMLNode elt; node_id = id; }
-  let make_dom ?(id = NoId) node = { elt = DomNode node; node_id = id; }
+  let make ?(id = NoId) elt = { elt = Lazy.lazy_from_val (TyXMLNode elt); node_id = id; }
+  let make_dom ?(id = NoId) node = { elt = Lazy.lazy_from_val (DomNode node); node_id = id; }
+  let make_lazy ?(id = NoId) lazy_elt =
+    let f () =
+       let elt = Lazy.force lazy_elt in
+       assert (elt.node_id = id);
+       Lazy.force elt.elt
+    in
+    { node_id = id; elt = Lazy.lazy_from_fun f }
+  let force_lazy { elt } = ignore (Lazy.force elt)
 
   let empty () = make Empty
 
@@ -71,10 +79,6 @@ module Xml = struct
   let event_handler_of_function (ev: #Dom_html.event Js.t -> unit) =
     Caml (CE_client_closure (Obj.magic ev))
 
-  (* Deprecated: Html5.F.a_on* functions are redefinied on the client
-     to call event_handler_of_function. *)
-  let event_of_function ev = ev
-
   let end_re = Regexp.regexp_string "]]>"
 
   let make_node_name =
@@ -106,6 +110,27 @@ module Xml = struct
       "\n/* <![CDATA[ */\n" ^ Regexp.global_replace end_re s "" ^ "\n/* ]]> */\n" in
     encodedpcdata s'
 
+  let set_classes node_id = function
+    | Empty
+    | Comment _
+    | EncodedPCDATA _
+    | PCDATA _
+    | Entity _ as e -> e
+    | Leaf (ename, attribs) ->
+      Leaf (ename, filter_class_attribs node_id attribs)
+    | Node (ename, attribs, sons) ->
+      Node (ename, filter_class_attribs node_id attribs, sons)
+
+  let set_classes_of_elt elt =
+     match Lazy.force elt.elt with
+      | DomNode _ -> failwith "Eliom_content_core.set_classes_of_elt"
+      | TyXMLNode econtent ->
+          { elt with elt = Lazy.lazy_from_val (TyXMLNode (set_classes elt.node_id econtent)) }
+
+  let string_of_node_id = function
+    | NoId -> "NoId"
+    | ProcessId s -> "ProcessId "^s
+    | RequestId s -> "RequestId "^s
 
 end
 
@@ -361,9 +386,47 @@ module Html5 = struct
     let string_of_id x = x
   end
 
+  module Custom_data = struct
+
+    type 'a t = {
+      name : string;
+      to_string : 'a -> string;
+      of_string : string -> 'a;
+      default : 'a option;
+    }
+
+    let create ~name ?default ~to_string ~of_string () =
+      { name ; of_string ; to_string; default }
+
+    let create_json ~name ?default typ =
+      { name ; of_string = of_json ~typ ; to_string = to_json ~typ; default }
+
+    let attrib custom_data value =
+      F.a_user_data
+        custom_data.name
+        (custom_data.to_string value)
+
+    let attribute_name name =
+      "data-"^name
+
+    let get_dom (element : Dom_html.element Js.t) custom_data =
+      Js.Opt.case
+        (element##getAttribute(Js.string (attribute_name custom_data.name)))
+        (fun () ->
+           match custom_data.default with
+             | Some value -> value
+             | None -> raise Not_found)
+        (fun str -> custom_data.of_string (Js.to_string str))
+
+    let set_dom element custom_data value =
+      element##setAttribute(Js.string (attribute_name custom_data.name),
+                            Js.string (custom_data.to_string value))
+
+  end
+
   module Of_dom = struct
     let rebuild_xml (node: 'a Js.t) : 'a F.elt =
-      Obj.magic { Xml.elt = Xml.DomNode (node :> Dom.node Js.t); node_id = Xml.NoId }
+      Obj.magic { Xml.elt = Lazy.lazy_from_val (Xml.DomNode (node :> Dom.node Js.t)); node_id = Xml.NoId }
     let of_element : Dom_html.element Js.t -> 'a elt = rebuild_xml
     let of_html : Dom_html.htmlElement Js.t -> Html5_types.html elt = rebuild_xml
     let of_head : Dom_html.headElement Js.t -> Html5_types.head elt = rebuild_xml
@@ -411,6 +474,8 @@ module Html5 = struct
     let of_iFrame : Dom_html.iFrameElement Js.t -> Html5_types.iframe elt = rebuild_xml
   end
 
+  let set_classes_of_elt elt = F.tot (X.set_classes_of_elt (F.toelt elt))
+
 end
 
 
