@@ -103,7 +103,18 @@ let fast_select_nodes root =
   a_nodeList, form_nodeList, process_node_nodeList, closure_nodeList
 
 let slow_has_classes (node:Dom_html.element Js.t) =
-  let classes = Js.str_array (node##className##split(Js.string " ")) in
+  let classes =
+    (* IE<9: className is not set after change_page; getAttribute("class")
+       does not work for the initial document *)
+    let str =
+      if node##className = Js.string "" then
+        Js.Opt.get (node##getAttribute(Js.string "class"))
+          (fun () -> Js.string "")
+      else
+        node##className
+    in
+    Js.str_array str##split(Js.string " ")
+  in
   let found_call_service = ref false in
   let found_process_node = ref false in
   let found_closure = ref false in
@@ -298,6 +309,9 @@ let copy_element (e:Dom.element Js.t)
     (registered_process_node:(Js.js_string Js.t -> bool)): Dom_html.element Js.t =
   let rec aux (e:Dom.element Js.t) =
     let copy = Dom_html.document##createElement(e##tagName) in
+    (* IE<9: Copy className seperatly, it's not updated when displayed *)
+    Js.Opt.iter (Dom_html.CoerceTo.element e)
+      (fun e -> copy##className <- e##className);
     let node_id = Js.Opt.to_option
       (e##getAttribute(Js.string Eliom_lib_base.RawXML.node_id_attrib)) in
     match node_id with
@@ -423,7 +437,7 @@ exception Incorrect_url
 let parse_absolute ~prefix href =
   match Regexp.search absolute_re href 0 with
   | Some (i, _) when i=0 -> (* absolute URL -> do not rewrite *) href
-  | _ -> 
+  | _ ->
     match Regexp.search absolute_re2 href 0 with
       | Some (i, res) when i = 0 ->
         (match Regexp.matched_group res 1 with
@@ -545,26 +559,33 @@ let max_preload_depth = ref 4
 
 let build_style (e, css) =
   lwt css = rewrite_css ~max:!max_preload_depth css in
-  lwt css =
+  (* lwt css = *)
     Lwt_list.map_p
       (fun (media, css) ->
-         let style = Dom_html.createStyle Dom_html.document in
-         style##_type <- Js.string "text/css";
-         style##media <- media;
-         style##innerHTML <- Js.string css;
-         Lwt.return (style :> Dom.node Js.t))
-      css in
-  (* Noscript is used to group style. It's ignored by the parser when
-     scripting is enabled, but does not seems to be ignore when
-     inserted as a DOM element. *)
-  let node = Dom_html.createNoscript Dom_html.document in
-  List.iter (Dom.appendChild node) css;
-  Lwt.return (e, node)
+        let style = Dom_html.createStyle Dom_html.document in
+        style##_type <- Js.string "text/css";
+        style##media <- media;
+        (* IE8: Assigning to style##innerHTML results in "Unknown runtime error" *)
+        let styleSheet = Js.Unsafe.(get style (Js.string "styleSheet")) in
+        if styleSheet != Js.undefined then
+          Js.Unsafe.(set styleSheet (Js.string "cssText") (Js.string css))
+        else
+          style##innerHTML <- Js.string css;
+        Lwt.return (e, (style :> Dom.node Js.t)))
+      css
+  (* IE8 doesn't allow appendChild on noscript-elements *)
+  (* (\* Noscript is used to group style. It's ignored by the parser when *)
+  (*    scripting is enabled, but does not seems to be ignore when *)
+  (*    inserted as a DOM element. *\) *)
+  (* let node = Dom_html.createNoscript Dom_html.document in *)
+  (* List.iteri (fun i x -> debug "HOC 3.%i" i; Dom.appendChild node x) css; *)
+  (* Lwt.return (e, node )*)
 
 let preload_css (doc : Dom_html.element Js.t) =
   if !Eliom_config.debug_timings then
     Firebug.console##time(Js.string "preload_css (fetch+rewrite)");
   lwt css = Lwt_list.map_p build_style (fetch_linked_css (get_head doc)) in
+  let css = List.concat css in
   List.iter (fun (e, css) ->
                try Dom.replaceChild (get_head doc) css e
                with _ ->
@@ -662,11 +683,10 @@ let onclick_on_body_handler event =
         (Js.Unsafe.variable "window")##eliomLastButton <- None);
   Js._true
 
-let add_formdata_hack_onclick_handler _ =
+let add_formdata_hack_onclick_handler () =
   ignore (Dom_html.addEventListener ( Dom_html.window##document##body )
             Dom_html.Event.click ( Dom_html.handler onclick_on_body_handler )
-            Js._true : Dom_html.event_listener_id);
-  true
+            Js._true : Dom_html.event_listener_id)
 
 (* END FORMDATA HACK *)
 
