@@ -44,7 +44,6 @@ type optgroup_attrib_t
 type option_attrib_t
 
 type input_type_t
-type raw_input_type_t
 type button_type_t
 
 type for_attrib
@@ -88,7 +87,7 @@ type for_attrib
     {- the [Host] http header of the current request if available;}
     {- the attribute [defaulthostname] of [<host>] tag in
     configuration file or the machine hostname in any other case.}}
-7    - [port] is:
+    - [port] is:
     {ul {- the optional parameter [~port] if given;}
     {- the attribute [defaulthttpsport] (resp. [defaulthttpport]) of [<host>] tag
     in configuration file or [443] (resp. 80) if [protocol] is [https] (resp. [http]) and
@@ -111,7 +110,7 @@ type for_attrib
 
     The optional parameter [nl_params] allows one to add non localized
     GET parameter to the URL.  See the eliom manual for more
-    information about {% <<a_manual chapter="params"
+    information about {% <<a_manual chapter="server-params"
     fragment="nonlocalizedparameters"|non localized parameters>>%}.
 *)
 val make_uri :
@@ -162,7 +161,7 @@ val make_string_uri :
 (** The function [uri_of_string f] returns a URI whose content is
     equivalent to [f ()].
 
-    For XML tree build with TyXML, like {!Html5}, {!Xhtml} or
+    For XML tree build with TyXML, like {!Html5} or
     {!Svg.F}, the function [f] is applied each time the XML tree is
     sent to the client (either as page content or as a marshalled
     OCaml value). Hence, the function is always evaluated in the
@@ -460,7 +459,7 @@ val user_type_input : ('a -> string) ->
 (** Creates an untyped [<input>] tag. You may use the name you want
     (for example to use with {!Eliom_parameter.any}). *)
 val raw_input :
-  ?a:input_attrib_t -> input_type:raw_input_type_t ->
+  ?a:input_attrib_t -> input_type:input_type_t ->
   ?name:string -> ?value:string -> unit -> input_elt
 
 (** Creates an [<input>] tag for sending a file *)
