@@ -55,11 +55,12 @@ end
 (*****************************************************************************)
 (*VVV Do not forget to change the version number
   when the internal format change!!! *)
-let persistent_cookie_table_version = "_v4" 
+let persistent_cookie_table_version = "_v5"
 (* v2 introduces session groups *)
 (* v3 introduces tab sessions *)
 (* v4 introduces group tables *)
-  
+(* v5 removes secure scopes *)
+
 let eliom_persistent_cookie_table =
   "eliom_persist_cookies"^persistent_cookie_table_version
 
@@ -68,18 +69,13 @@ let servicecookiename = "eliomservicesession|"
 (* must not be a prefix of the following and vice versa (idem for data) *)
 let persistentcookiename = "eliompersistentsession|"
 
-(* the same, secure: *)
-let sdatacookiename = "Seliomdatasession|"
-let sservicecookiename = "Seliomservicesession|"
-let spersistentcookiename = "Seliompersistentsession|"
-
 (*****************************************************************************)
 
 let eliom_link_too_old : bool Polytables.key = Polytables.make_key ()
 (** The coservice does not exist any more *)
 
 let eliom_service_session_expired : 
-    (fullsessionname list * fullsessionname list) Polytables.key = 
+    (full_state_name list * full_state_name list) Polytables.key = 
   Polytables.make_key ()
 (** If present in request data,  means that
     the service session cookies does not exist any more.
@@ -118,8 +114,7 @@ module SessionCookies =
 
 (* session groups *)
 type 'a sessgrp =
-    (string * cookie_scope
-     * (string, Ip_address.t) leftright)
+    (string * cookie_level * (string, Ip_address.t) leftright)
     (* The full session group is the triple
        (site_dir_string, scope, session group name).
        The scope is the scope of group members (`Session by default).
@@ -127,11 +122,11 @@ type 'a sessgrp =
        we limit the number of sessions by IP address. *)
 type perssessgrp = string (* same triple, marshaled *)
 
-let make_persistent_full_group_name ~cookie_scope site_dir_string = function
+let make_persistent_full_group_name ~cookie_level site_dir_string = function
   | None -> None
   | Some g ->
     Some (Marshal.to_string
-            (site_dir_string, cookie_scope, Left g) [])
+            (site_dir_string, cookie_level, Left g) [])
 
 let getperssessgrp a = Marshal.from_string a 0
 
@@ -154,7 +149,7 @@ type 'a one_service_cookie_info =
                                     ref towards cookie table
                                   *);
      sc_cookie_exp:cookie_exp ref (* cookie expiration date to set *);
-     sc_session_group: cookie_scope sessgrp ref
+     sc_session_group: cookie_level sessgrp ref
        (* session group *);
      mutable sc_session_group_node:string Ocsigen_cache.Dlist.node;
    }
@@ -171,7 +166,7 @@ type one_data_cookie_info =
                                            ref towards cookie table
                                          *);
      dc_cookie_exp:cookie_exp ref       (* cookie expiration date to set *);
-     dc_session_group: cookie_scope sessgrp ref (* session group *);
+     dc_session_group: cookie_level sessgrp ref (* session group *);
      mutable dc_session_group_node:string Ocsigen_cache.Dlist.node;
    }
 
@@ -199,7 +194,7 @@ type 'a cookie_info1 =
     )
       (* This one is not lazy because we must check all service sessions
          at each request to find the services *)
-      Fullsessionname_Table.t ref (* The key is the full session name *) *
+      Full_state_name_table.t ref (* The key is the full session name *) *
 
     (* in memory data sessions: *)
       (string option            (* value sent by the browser *)
@@ -216,7 +211,7 @@ type 'a cookie_info1 =
       (* Lazy because we do not want to ask the browser to unset the cookie
          if the cookie has not been used, otherwise it is impossible to
          write a message "Your session has expired" *)
-      Fullsessionname_Table.t ref (* The key is the full session name *) *
+      Full_state_name_table.t ref (* The key is the full session name *) *
 
       (* persistent sessions: *)
       ((string                  (* value sent by the browser *) *
@@ -236,7 +231,7 @@ type 'a cookie_info1 =
             For both of them, ask the browser to remove the cookie.
           *)
       ) Lwt.t Lazy.t
-      Fullsessionname_Table.t ref
+      Full_state_name_table.t ref
 
 
 type 'a cookie_info =
@@ -247,12 +242,12 @@ type 'a cookie_info =
 
 (* non persistent cookies for services *)
 type 'a servicecookiestablecontent =
-    (fullsessionname     (* fullsessname *) *
+    (full_state_name *
      'a                  (* session table *) *
      float option ref    (* expiration date by timeout
                             (server side) *) *
      timeout ref         (* user timeout *) *
-     cookie_scope sessgrp ref   (* session group *) *
+     cookie_level sessgrp ref   (* session group *) *
      string Ocsigen_cache.Dlist.node (* session group node *))
 
 type 'a servicecookiestable = 'a servicecookiestablecontent SessionCookies.t
@@ -267,11 +262,11 @@ type 'a servicecookiestable = 'a servicecookiestablecontent SessionCookies.t
 
 (* non persistent cookies for in memory data *)
 type datacookiestablecontent =
-    (fullsessionname         (* fullsessname *) *
+    (full_state_name *
      float option ref        (* expiration date by timeout
                                 (server side) *) *
      timeout ref             (* user timeout *) *
-     cookie_scope sessgrp ref   (* session group *) *
+     cookie_level sessgrp ref   (* session group *) *
      string Ocsigen_cache.Dlist.node (* session group node *))
 
 type datacookiestable = datacookiestablecontent SessionCookies.t
@@ -367,6 +362,8 @@ type node_info = {
   mutable ni_sent : bool;
 }
 
+module Hier_set = String.Set
+
 type server_params =
     {sp_request: Ocsigen_extensions.request;
      sp_si: sess_info;
@@ -379,10 +376,10 @@ type server_params =
      mutable sp_client_appl_name: string option; (* The application name,
                                                     as sent by the browser *)
      sp_suffix: Url.path option (* suffix *);
-     sp_fullsessname: fullsessionname option (* the name of the session
-                                                to which belong the service
-                                                that answered
-                                                (if it is a session service) *);
+     sp_full_state_name: full_state_name option
+                             (* the name of the session
+                                to which belong the service that answered
+                                (if it is a session service) *);
      sp_client_process_info: client_process_info;
     }
 
@@ -485,21 +482,24 @@ and sitedata =
    mutable servtimeout: 
      (float option * bool) option * 
      (float option * bool) option * 
-     ((fullsessionname * (float option * bool)) list);
+     ((full_state_name * (float option * bool)) list);
    mutable datatimeout: 
      (float option * bool) option * 
      (float option * bool) option * 
-     ((fullsessionname * (float option * bool)) list);
+     ((full_state_name * (float option * bool)) list);
    mutable perstimeout: 
      (float option * bool) option *
      (float option * bool) option * 
-     ((fullsessionname * (float option * bool)) list);
+     ((full_state_name * (float option * bool)) list);
 
    site_value_table : Polytables.t; (* table containing evaluated
-					    lazy site values *)
+				       lazy site values *)
 
-   mutable registered_scope_names: String.Set.t;
+   mutable registered_scope_hierarchies: Hier_set.t;
 
+   (* All services, and state data are stored in these tables,
+      for scopes session and client process.
+      The scope is registered in the full session name. *)
    global_services: tables; (* global service table *)
    session_services: tables servicecookiestable;
    (* cookie table for services (tab and browser sessions) *)
@@ -535,21 +535,28 @@ and dlist_ip_table = (page_table ref * page_table_key, na_key_serv)
 
 (*****************************************************************************)
 
-let make_full_cookie_name a b = a^b
-
-let make_fullsessname2 site_dir_string (scope:[< user_scope ]) : fullsessionname =
-  let cookie_scope = cookie_scope_of_user_scope scope in
-  let state_name = scope_name_of_scope scope in
-  let name = match state_name with
-    | `Default_ref_name -> "ref|"
-    | `Default_comet_name -> "comet|"
-    | `String s -> "|"^s
+let make_full_cookie_name cookieprefix (cookie_scope, secure, site_dir_string) =
+  let scope_hier = scope_hierarchy_of_scope cookie_scope in
+  let secure = if secure then "S|" else "|" in
+  let hier1, hiername = match scope_hier with
+    | User_hier hiername -> "||", hiername 
+    | Default_ref_hier -> "|ref|", ""
+    | Default_comet_hier -> "|comet|", ""
   in
-  ((cookie_scope :> cookie_scope), site_dir_string^name)
-(* Warning: do not change this without modifying Eliom_state.Admin *)
+  let s = String.concat ""
+    [cookieprefix; secure; site_dir_string; hier1; hiername]
+  in
+  s
 
-let make_fullsessname ~sp (scope:[< user_scope ]) =
-  make_fullsessname2 sp.sp_sitedata.site_dir_string scope
+let make_full_state_name2
+    site_dir_string secure ~(scope:[< user_scope ]) : full_state_name =
+  (* The information in the cookie name, without the kind of session *)
+  ((scope :> user_scope),
+   secure,
+   site_dir_string)
+
+let make_full_state_name ~sp ~secure ~(scope:[< user_scope ]) =
+  make_full_state_name2 sp.sp_sitedata.site_dir_string secure scope
 
 let get_cookie_info sp = function
   | `Session -> sp.sp_cookie_info
@@ -563,7 +570,7 @@ let make_server_params
     sitedata
     (ri, si, all_cookie_info, all_tab_cookie_info, user_tab_cookies)
     suffix
-    fullsessname =
+    full_state_name =
   let appl_name =
     try
       Some
@@ -592,7 +599,7 @@ let make_server_params
     sp_user_tab_cookies = user_tab_cookies;
     sp_client_appl_name = appl_name;
     sp_suffix = suffix;
-    sp_fullsessname = fullsessname;
+    sp_full_state_name = full_state_name;
     sp_client_process_info = cpi;
   }
 
@@ -614,37 +621,43 @@ let sp_of_option sp =
 (* Scope registration                                                        *)
 (*****************************************************************************)
 
-let global : global_scope = `Global
-let site : site_scope = `Site
-let session_group : session_group_scope = `Session_group `Default_ref_name
-let session : session_scope = `Session `Default_ref_name
-let client_process : client_process_scope = `Client_process `Default_ref_name
-let comet_client_process : client_process_scope = `Client_process `Default_comet_name
-let request : request_scope = `Request
+let global_scope : global_scope = `Global
+let site_scope : site_scope = `Site
 
-let registered_scope_names = ref String.Set.empty
+let default_group_scope : session_group_scope = `Session_group Default_ref_hier
+let default_session_scope : session_scope = `Session Default_ref_hier
+let default_process_scope : client_process_scope = `Client_process Default_ref_hier
 
-let register_scope_name (name:string) =
+let comet_client_process_scope : client_process_scope = `Client_process Default_comet_hier
+let request_scope : request_scope = `Request
+
+let registered_scope_hierarchies = ref Hier_set.empty
+
+let register_scope_hierarchy (name:string) =
   match get_sp_option () with
     | None ->
-      if String.Set.mem name !registered_scope_names
-      then failwith (Printf.sprintf "the scope %s have already been registered" name)
-      else registered_scope_names := String.Set.add name !registered_scope_names
+      if Hier_set.mem name !registered_scope_hierarchies
+      then failwith (Printf.sprintf "the scope hierarchy %s has already been registered" name)
+      else registered_scope_hierarchies :=
+        Hier_set.add name !registered_scope_hierarchies
     | Some sp ->
-      if String.Set.mem name !registered_scope_names ||
-	String.Set.mem name sp.sp_sitedata.registered_scope_names
-      then failwith (Printf.sprintf "the scope %s have already been registered" name)
-      else sp.sp_sitedata.registered_scope_names <- String.Set.add name sp.sp_sitedata.registered_scope_names
+      if Hier_set.mem name !registered_scope_hierarchies ||
+	Hier_set.mem name sp.sp_sitedata.registered_scope_hierarchies
+      then failwith (Printf.sprintf "the scope hierarchy %s has already been registered" name)
+      else sp.sp_sitedata.registered_scope_hierarchies <-
+        Hier_set.add name sp.sp_sitedata.registered_scope_hierarchies
 
-let create_scope_name name : scope_name =
-  register_scope_name name;
-  `String name
+let create_scope_hierarchy name : scope_hierarchy =
+  register_scope_hierarchy name;
+  User_hier name
 
-let list_scope_names () =
+let list_scope_hierarchies () =
   let sp = get_sp () in
-  `Default_comet_name::`Default_ref_name::
-    ( List.map (fun s -> `String s) (String.Set.elements !registered_scope_names)
-      @ List.map (fun s -> `String s) (String.Set.elements sp.sp_sitedata.registered_scope_names) )
+  Default_comet_hier::Default_ref_hier::
+    (List.map (fun s -> User_hier s)
+       (Hier_set.elements !registered_scope_hierarchies)
+     @ List.map (fun s -> User_hier s)
+       (Hier_set.elements sp.sp_sitedata.registered_scope_hierarchies) )
 
 (*****************************************************************************)
 (* The current registration directory *)
@@ -897,21 +910,46 @@ let split_nl_prefix_param =
     in
     aux [] String.Table.empty l
 
-let getcookies cookie_scope cookiename cookies =
-  let length = String.length cookiename in
+(* The cookie name is 
+
+sessionkind|S?|sitedirstring|"ref" ou "comet" ou ""|hiername
+
+*)
+
+let full_state_name_of_cookie_name cookie_level cookiename =
+  let pref, cookiename = Ocsigen_lib.String.sep '|' cookiename in
+  let secure, cookiename = Ocsigen_lib.String.sep '|' cookiename in
+  let sitedirstring, cookiename = Ocsigen_lib.String.sep '|' cookiename in
+  let hier1, hiername = Ocsigen_lib.String.sep '|' cookiename in
+  let secure = secure = "S" in
+  let sc_hier = match hier1 with
+    | "" -> Eliom_common_base.User_hier hiername
+    | "ref" -> Eliom_common_base.Default_ref_hier
+    | "comet" -> Eliom_common_base.Default_comet_hier
+    | _ -> raise Not_found
+  in
+  match cookie_level with
+    | `Session -> (`Session sc_hier, secure, sitedirstring)
+    | `Client_process -> (`Client_process sc_hier, secure, sitedirstring)
+
+let getcookies secure cookie_level cookienamepref cookies =
+  let length = String.length cookienamepref in
   let last = length - 1 in
   CookiesTable.fold
     (fun name value beg ->
-      if String.first_diff cookiename name 0 last = length
+      if String.first_diff cookienamepref name 0 last = length
       then
-        Fullsessionname_Table.add
-          (cookie_scope, (String.sub name length ((String.length name) - length)))
-          value
-          beg
+        try
+          let (_, sec, _) as expcn =
+            full_state_name_of_cookie_name cookie_level name in
+          if sec = secure
+          then Full_state_name_table.add expcn value beg
+          else beg
+        with Not_found -> beg
       else beg
     )
     cookies
-    Fullsessionname_Table.empty
+    Full_state_name_table.empty
 
 (* Remove all parameters whose name starts with pref *)
 let remove_prefixed_param pref l =
@@ -1056,17 +1094,17 @@ let get_session_info req previous_extension_err =
 
   let browser_cookies = Lazy.force ri.Ocsigen_extensions.ri_cookies in
 
-  let data_cookies = getcookies `Session datacookiename browser_cookies in
-  let service_cookies = getcookies `Session servicecookiename browser_cookies in
-  let persistent_cookies = getcookies `Session persistentcookiename browser_cookies in
+  let data_cookies = getcookies false `Session datacookiename browser_cookies in
+  let service_cookies = getcookies false `Session servicecookiename browser_cookies in
+  let persistent_cookies = getcookies false `Session persistentcookiename browser_cookies in
   
   let secure_cookie_info =
     if ri.Ocsigen_extensions.ri_ssl
     then
-      let sdata_cookies = getcookies `Session sdatacookiename browser_cookies in
-      let sservice_cookies = getcookies `Session sservicecookiename browser_cookies in
+      let sdata_cookies = getcookies true `Session datacookiename browser_cookies in
+      let sservice_cookies = getcookies true `Session servicecookiename browser_cookies in
       let spersistent_cookies =
-        getcookies `Session spersistentcookiename browser_cookies
+        getcookies true `Session persistentcookiename browser_cookies
       in
       Some (sservice_cookies, sdata_cookies, spersistent_cookies)
     else None
@@ -1184,16 +1222,16 @@ let get_session_info req previous_extension_err =
          nl_get_params String.Table.empty)
   in
 
-  let data_cookies_tab = getcookies `Client_process datacookiename tab_cookies in
-  let service_cookies_tab = getcookies `Client_process servicecookiename tab_cookies in
-  let persistent_cookies_tab = getcookies `Client_process persistentcookiename tab_cookies in
+  let data_cookies_tab = getcookies false `Client_process datacookiename tab_cookies in
+  let service_cookies_tab = getcookies false `Client_process servicecookiename tab_cookies in
+  let persistent_cookies_tab = getcookies false `Client_process persistentcookiename tab_cookies in
 
   let secure_cookie_info_tab =
     if ri.Ocsigen_extensions.ri_ssl
     then
-      let sdata_cookies = getcookies `Client_process sdatacookiename tab_cookies in
-      let sservice_cookies = getcookies `Client_process sservicecookiename tab_cookies in
-      let spersistent_cookies = getcookies `Client_process spersistentcookiename tab_cookies in
+      let sdata_cookies = getcookies true `Client_process datacookiename tab_cookies in
+      let sservice_cookies = getcookies true `Client_process servicecookiename tab_cookies in
+      let spersistent_cookies = getcookies true `Client_process persistentcookiename tab_cookies in
       Some (sservice_cookies, sdata_cookies, spersistent_cookies)
     else None
   in
@@ -1307,7 +1345,7 @@ let create_persistent_table name =
   Ocsipersist.open_table name
 
 let persistent_cookies_table :
-    (fullsessionname * float option * timeout * perssessgrp option)
+    (full_state_name * float option * timeout * perssessgrp option)
     Ocsipersist.table Lazy.t =
   lazy (create_persistent_table eliom_persistent_cookie_table)
 (* Another tables, containing the session info for each cookie *)
