@@ -19,12 +19,12 @@
  *)
 
 (** Eliom services registration for various kinds of
-    page content: Eliom application, valid {!Html5} or {!Xhtml},
+    page content: Eliom application, valid {!Html5},
     actions, redirections, static files, … *)
 
 (** See the Eliom manual for
-    more information on {% <<a_manual chapter="services" | services
-    >>%} and {% <<a_manual chapter="outputs" | predefined outputs
+    more information on {% <<a_manual chapter="server-services" | services
+    >>%} and {% <<a_manual chapter="server-outputs" | predefined outputs
     >>%}. *)
 
 (** {% <<outline>> %}*)
@@ -71,8 +71,7 @@ type http_service = [ `Http ]
     means the returned content must be interpreted in the browser as
     stated by the content-type header. This is most common return type
     for an eliom service, see for example {!Html5},
-    {!Xhtml}, {!CssText}, {!File},
-    {!Redirection}, ….*)
+    {!CssText}, {!File}, {!Redirection}, ….*)
 type browser_content = [ `Browser ]
 
 (** The type [block_content] is a refinement of {!http_service} to be
@@ -121,7 +120,7 @@ type non_caml_service = [ appl_service | http_service ]
 (** {3 Module signature} *)
 
 (** Abstract signature for service registration functions. For
-    concrete instance see {!Html5}, {!Xhtml}, {!CssText}, {!File},
+    concrete instance see {!Html5}, {!CssText}, {!File},
     {!Redirection}, … *)
 module type Registration = sig
   type page
@@ -145,19 +144,6 @@ module Html5 : sig
   include "sigs/eliom_html5_reg.mli"
 end
 
-(** {2 Using XHTML with services } *)
-
-(** Eliom service registration for XHTML page. This
-    an instance the {!Registration} abstract signatures. *)
-module Xhtml : sig
-  include "sigs/eliom_xhtml_reg.mli"
-end
-
-(** Eliom service registration for services that returns XHTML
-    page. This is a subset of the {!Xhtml} module and an instance of
-    the {!Registration} abstract signature. *)
-module Xhtml_registration : "sigs/eliom_xhtml_reg.mli"
-
 (** {2 Eliom client/server applications} *)
 
 (** Signature for application creation. *)
@@ -206,6 +192,9 @@ module type ELIOM_APPL = sig
       client.}  *)
   val application_name : string
 
+  (** Checks during a request whether it is the initial request of the
+      client process in this Eliom application. *)
+  val is_initial_request : unit -> bool
 
   (** The type [appl] is an abstract type for identifying an
       application. It usually used a phantom parameter for
@@ -231,7 +220,7 @@ module type TMPL_PARAMS = sig
   type t
   val name: string
   val make_page: t -> Html5_types.html Eliom_content_core.Html5.elt Lwt.t
-  val update: t -> Dom_html.event Xml.caml_event_handler
+  val update: t -> unit client_value
 end
 
 module Eliom_tmpl (Appl : ELIOM_APPL) (Tmpl_param : TMPL_PARAMS): sig
@@ -251,7 +240,7 @@ end
     signature.
 
     For Eliom application, prefers {!Ocaml} services to send page
-    fragment.
+    fragments.
 *)
 module Flow5 : "sigs/eliom_reg.mli"
   subst type page    := Html5_types.flow5 Eliom_content_core.Html5.elt list
@@ -259,22 +248,6 @@ module Flow5 : "sigs/eliom_reg.mli"
   and type return  := http_service
   and type result  := (block_content, http_service) kind
 
-(** Deprecated alias for {!Flow5}. *)
-module Block5 : "sigs/eliom_reg.mli"
-  subst type page    := Html5_types.flow5 Eliom_content_core.Html5.elt list
-  and type options := unit
-  and type return  := http_service
-  and type result  := (block_content, http_service) kind
-
-(** Eliom service registration and forms creation for fragment of
-    XHTML page. This is an instance of the {!Registration} abstract
-    signature. *)
-module Block : "sigs/eliom_reg.mli"
-  subst type page    := Xhtml_types.body_content Eliom_content_core.Xhtml.F.elt list
-  and type options := unit
-  and type return  := http_service
-  and type result  := (block_content, http_service) kind
-
 (** Eliom service registration for services that returns fragment of
     TyXML's tree. The returned module is an instance of the
     {!Registration} abstract signature. *)
@@ -307,7 +280,7 @@ end
 
 (** Eliom service registration for services that returns CSS. The page
     content is a [string] that must contains valid CSS and the content
-    type is always [text/css]. The option is the optionnal
+    type is always [text/css]. The option is the optional
     "Cache-policy: max-age" header value to be sent.
 
     This an instance of the {!Registration} abstract signature.
@@ -322,7 +295,7 @@ module CssText : "sigs/eliom_reg.mli"
 
 (** Eliom service registration for services that only execute
     actions. See the Eliom manual for more information about {%
-    <<a_manual chapter="outputs" fragment="actions"|Actions
+    <<a_manual chapter="server-outputs" fragment="actions"|Actions
     outputs>>%}.
 
     If you give the optional parameter [~options:`NoReload] to the
@@ -346,7 +319,7 @@ module Unit : "sigs/eliom_reg.mli"
 
 (** Eliom service registration for services that returns a redirections
     towards another service. See the Eliom manual for more
-    information about {% <<a_manual chapter="outputs"
+    information about {% <<a_manual chapter="server-outputs"
     fragment="redirections"|Redirections outputs>>%}.
 
     The default returned HTTP code is [302 Found]. You could use the
@@ -357,7 +330,7 @@ module Unit : "sigs/eliom_reg.mli"
     - [`SeeOther] to return [303 See Other].
     - [`NotNodifed] to return [304 Not Modified].
     - [`UseProxy] to return [305 Use Proxy].
-    - [`TemporaryRedirect] to return [301 Temporary Redirect].
+    - [`TemporaryRedirect] to return [307 Temporary Redirect].
 
 *)
 module Redirection : "sigs/eliom_reg_alpha_return.mli"
@@ -377,7 +350,7 @@ module Redirection : "sigs/eliom_reg_alpha_return.mli"
 
 (** Eliom service registration for services that returns a redirections
     towards a string-URL. See the Eliom manual for more information
-    about {% <<a_manual chapter="outputs"
+    about {% <<a_manual chapter="server-outputs"
     fragment="redirections"|Redirections outputs>>%}. The URL given
     must be an absolute URI.
 
@@ -400,13 +373,13 @@ module String_redirection : "sigs/eliom_reg.mli"
 
 (** Eliom service registration for services that returns file
     contents. The page is the name of the file to send. See the Eliom
-    manual for more information on {% <<a_manual chapter="outputs"
+    manual for more information on {% <<a_manual chapter="server-outputs"
     fragment="eliomfiles"|how to send files with Eliom>>%}. *)
 module File : sig
 
   (** The function [check_file file] is true if [File.send file]
       would effectively return the file (i.e. the file is present and
-      readable.) The option is the optionnal "Cache-policy: max-age"
+      readable.) The option is the optional "Cache-policy: max-age"
       header value to be sent.
   *)
 
@@ -433,7 +406,7 @@ module Ocaml : "sigs/eliom_reg_simpl.mli"
 (** Eliom service registration for services that choose dynamically
     what they want to send. The content is created using for example
     {!Html5.send} or {!String.send} functions. See the Eliom manual
-    for more information about {% <<a_manual chapter="outputs"
+    for more information about {% <<a_manual chapter="server-outputs"
     fragment="any"|services that choose dynamically what they want to
     send>>%} *)
 module Any : "sigs/eliom_reg_alpha_return.mli"
@@ -444,7 +417,7 @@ module Any : "sigs/eliom_reg_alpha_return.mli"
 
 (** The function [appl_self_redirect send page] is an helper function
     required for defining {!Any} service usable inside an Eliom
-    application ({!App}). It allows to cast an Eliom senders
+    application ({!App}). It allows casting an Eliom senders
     that do not returns {!application_content} (like {!File.send},
     {!String.send}, ...) into a senders returns
     {!application_content}.
@@ -463,7 +436,7 @@ val appl_self_redirect :
 (** Eliom service registration for services that returns "byte"-string
     contents. The page content is a couple [(raw_content,
     content_type)]. See also {!Streamlist} for another kind of service
-    that returns "byte" contents. The option is the optionnal
+    that returns "byte" contents. The option is the optional
     "Cache-policy: max-age" header value to be sent.
 
     This an instance of the {!Registration} abstract signature.
@@ -474,13 +447,6 @@ module String : "sigs/eliom_reg.mli"
   and type return  := http_service
   and type result  := (unknown_content, http_service) kind
 
-(** Deprecated alias for {!String}. *)
-module Text : "sigs/eliom_reg.mli"
-  subst type page  := string * string
-  and type options := int
-  and type return  := http_service
-  and type result  := (unknown_content, http_service) kind
-
 (** Eliom service registration for services that returns "byte"
     contents with {% <<a_api project="ocsigenserver" text="Ocsigen's
     streams"| module Ocsigen_stream>>%}. The page content is a couple
@@ -501,7 +467,7 @@ module Streamlist : "sigs/eliom_reg.mli"
 
 (** {2 Customizing registration} *)
 
-(** The [Customize] functor allows to specialize service registration
+(** The [Customize] functor allows specialization of service registration
     functions by customizing the page type. See the {% <<a_manual
     project="tutorial" chapter="interaction"| Eliom tutorial>>%} for
     example. *)
@@ -515,7 +481,7 @@ module Customize :
 
 (** {2 Using your own error pages} *)
 
-(** The [set_exn_handler handler] allows to redefines error pages:
+(** The [set_exn_handler handler] allows redefinition of error pages:
     [404] or any exception during page generation.
 
     Note that you should not catch every exception here since some Eliom
