@@ -106,9 +106,9 @@ let service_aux
       (match Eliom_common.global_register_allowed () with
       | Some get_current_sitedata ->
           let sitedata = get_current_sitedata () in
-          let path = 
+          let path =
             Url.remove_internal_slash
-              (Url.change_empty_list 
+              (Url.change_empty_list
                  (Url.remove_slash_at_beginning path))
           in
           let u = service_aux_aux
@@ -131,9 +131,9 @@ let service_aux
           raise (Eliom_common.Eliom_site_information_not_available
                    "service"))
   | Some sp ->
-      let path = 
+      let path =
         Url.remove_internal_slash
-          (Url.change_empty_list 
+          (Url.change_empty_list
              (Url.remove_slash_at_beginning path))
       in
       service_aux_aux
@@ -172,7 +172,7 @@ let default_csrf_scope = function
     (* We do not use the classical syntax for default
        value. Otherwise, the type for csrf_scope was:
        [< Eliom_common.user_scope > `Session] *)
-  | None -> `Session `Default_ref_name
+  | None -> `Session Eliom_common_base.Default_ref_hier
   | Some c -> (c :> [Eliom_common.user_scope])
 
 let coservice
@@ -211,15 +211,15 @@ let coservice
         get_or_post = `Get;
      };
    https = https || fallback.https;
-   keep_nl_params = match keep_nl_params with 
+   keep_nl_params = match keep_nl_params with
      | None -> fallback.keep_nl_params | Some k -> k;
  }
 (* Warning: here no GET parameters for the fallback.
    Preapply services if you want fallbacks with GET parameters *)
 
 
-let coservice' 
-    ?name 
+let coservice'
+    ?name
     ?(csrf_safe = false)
     ?csrf_scope
     ?csrf_secure
@@ -243,7 +243,7 @@ let coservice'
           max_use= max_use;
           timeout= timeout;
           pre_applied_parameters = String.Table.empty, [];
-          get_params_type = 
+          get_params_type =
             add_pref_params Eliom_common.na_co_param_prefix get_params;
           post_params_type = unit;
           kind = `Nonattached
@@ -267,7 +267,7 @@ let coservice'
 
 (****************************************************************************)
 (* Create a service with post parameters in the server *)
-let post_service_aux ~https ~fallback 
+let post_service_aux ~https ~fallback
     ?(keep_nl_params = `None) ?(priority = default_priority) ~post_params =
 (* Create a main service (not a coservice) internal, post only *)
 (* ici faire une vrification "duplicate parameter" ? *)
@@ -296,7 +296,7 @@ let post_service_aux ~https ~fallback
    service_mark = service_mark ();
  }
 
-let post_service ?(https = false) ~fallback 
+let post_service ?(https = false) ~fallback
     ?keep_nl_params ?priority ~post_params () =
   (* POST service without POST parameters means
      that the service will answer to a POST request only.
@@ -348,7 +348,7 @@ let post_coservice
      {k1 with
         att_kind = `Internal `Coservice;
         get_or_post = `Post;
-        post_name = 
+        post_name =
          (if csrf_safe
           then Eliom_common.SAtt_csrf_safe (uniqueid (),
                                             (csrf_scope:>Eliom_common.user_scope),
@@ -359,7 +359,7 @@ let post_coservice
                | Some name -> Eliom_common.SAtt_named name));
      };
    https = https;
-   keep_nl_params = match keep_nl_params with 
+   keep_nl_params = match keep_nl_params with
      | None -> fallback.keep_nl_params | Some k -> k;
  }
 (* It is not possible to make a post_coservice function
@@ -391,7 +391,7 @@ let post_coservice'
     get_params_type = unit;
     post_params_type = post_params;
     kind = `Nonattached
-      {na_name = 
+      {na_name =
           (if csrf_safe
            then Eliom_common.SNa_post_csrf_safe (uniqueid (),
                                                  (csrf_scope:>Eliom_common.user_scope),
@@ -482,7 +482,7 @@ let remove_service table service =
         let attserpost = get_post_name_ attser in
         let sgpt = get_get_params_type_ service in
         let sppt = get_post_params_type_ service in
-        Eliommod_services.remove_service table 
+        Eliommod_services.remove_service table
           (get_sub_path_ attser)
           {Eliom_common.key_state = (attserget, attserpost);
            Eliom_common.key_kind = key_kind}
@@ -526,44 +526,6 @@ let unregister ?scope ?secure service =
 
 
 
-(*****************************************************************************)
-(** {2 on_load and on_unload for App services } *)
-
-(* We keep them in rc because we want them to apply to the next page that
-   will be displayed. That is, event after an action or a (stateful)
-   redirection.
-*)
-
-let on_load_key : Dom_html.event Xml.caml_event_handler list Polytables.key = Polytables.make_key ()
-
-let get_onload sp =
-  let rc = Eliom_request_info.get_request_cache_sp sp in
-  try
-    List.rev (Polytables.get ~table:rc ~key:on_load_key)
-  with Not_found -> []
-
-let on_unload_key : Dom_html.event Xml.caml_event_handler list Polytables.key = Polytables.make_key ()
-
-let get_onunload sp =
-  let rc = Eliom_request_info.get_request_cache_sp sp in
-  try
-    List.rev (Polytables.get ~table:rc ~key:on_unload_key)
-  with Not_found -> []
-
-let onload s =
-  let rc = Eliom_request_info.get_request_cache () in
-  let s0 = try Polytables.get ~table:rc ~key:on_load_key
-    with Not_found -> []
-  in
-  Polytables.set ~table:rc ~key:on_load_key ~value:(s :: s0)
-
-let onunload s =
-  let rc = Eliom_request_info.get_request_cache () in
-  let s0 = try Polytables.get ~table:rc ~key:on_unload_key
-    with Not_found -> []
-  in
-  Polytables.set ~table:rc ~key:on_unload_key ~value:(s :: s0)
-
 (*****************************************************************************)
 let pre_wrap s =
   {s with
@@ -572,3 +534,131 @@ let pre_wrap s =
   }
 
 (* let wrap s = Eliom_types.wrap_parameters (pre_wrap s) *)
+
+(******************************************************************************)
+
+(* Global data *)
+
+let get_global_data, modify_global_data =
+  (* We have to classify global data from ocsigen extensions (no site
+     available) and eliommodules (site data available).
+     Furthermore, the Eliom services must only send global data from
+     ocsigen extensions and their own site.  *)
+  let global_data = ref String_map.empty in
+  let site_data = Eliom_reference.Volatile.eref ~scope:Eliom_common.site_scope String_map.empty in
+  let is_site_available () =
+    (* Matches valid states for Eliom_common.get_site_data *)
+    Eliom_common.(get_sp_option () <> None || during_eliom_module_loading ())
+  in
+  let get () =
+    if is_site_available () then
+      String_map.merge
+        (fun compilation_unit_id global site ->
+          match global, site with
+            | None, None -> assert false
+            | Some data, None | None, Some data -> Some data
+            | Some _, Some site_data ->
+              Printf.ksprintf Ocsigen_messages.errlog
+                "Compilation unit %s linked globally AND as Eliom module"
+                compilation_unit_id;
+              Some site_data)
+        !global_data
+        (Eliom_reference.Volatile.get site_data)
+    else
+      !global_data
+  in
+  let modify f =
+    if is_site_available () then
+      Eliom_reference.Volatile.modify site_data f
+    else
+      global_data := f !global_data
+  in
+  get, modify
+
+let current_server_section_data = ref []
+
+let get_compilation_unit_global_data compilation_unit_id =
+  if not (String_map.mem compilation_unit_id (get_global_data ())) then
+    ( let data = { server_sections_data = Queue.create (); client_sections_data = Queue.create () } in
+      ignore (modify_global_data (String_map.add compilation_unit_id data)) );
+  String_map.find compilation_unit_id (get_global_data ())
+
+let close_server_section ~compilation_unit_id =
+  let { server_sections_data } =
+    get_compilation_unit_global_data compilation_unit_id
+  in
+  Queue.push (List.rev !current_server_section_data)
+    server_sections_data;
+  current_server_section_data := []
+
+let close_client_section ~compilation_unit_id injection_data =
+  let { client_sections_data } =
+    get_compilation_unit_global_data compilation_unit_id
+  in
+  let injection_datum (injection_id, injection_value) =
+    { injection_id; injection_value }
+  in
+  Queue.push (List.map injection_datum injection_data)
+    client_sections_data
+
+let get_global_data () =
+  let on_injection_datum injection_datum =
+    let injection_value = injection_datum.injection_value () in
+    { injection_datum with injection_value  }
+  in
+  String_map.map
+    (fun compilation_unit_global_data ->
+       let client_sections_data = Queue.create () in
+       Queue.iter
+         (fun injection_data ->
+            Queue.push (List.map on_injection_datum injection_data)
+              client_sections_data)
+         compilation_unit_global_data.client_sections_data;
+       { compilation_unit_global_data with client_sections_data })
+    (get_global_data ())
+
+(* Request data *)
+
+let request_data : request_data Eliom_reference.Volatile.eref =
+  Eliom_reference.Volatile.eref ~scope:Eliom_common.request_scope []
+
+let get_request_data () =
+  List.rev (Eliom_reference.Volatile.get request_data)
+
+(* Register data *)
+
+let is_global = ref false
+
+let register_client_value_data ~closure_id ~instance_id ~args =
+  let client_value_datum = { closure_id; instance_id; args } in
+  if !is_global then
+    if Eliom_common.get_sp_option () = None then
+      current_server_section_data :=
+        client_value_datum :: !current_server_section_data
+    else
+      raise (Client_value_creation_invalid_context closure_id)
+  else
+    Eliom_reference.Volatile.modify request_data
+      (fun sofar -> client_value_datum :: sofar)
+
+(* Syntax helpers *)
+
+module Syntax_helpers = struct
+
+  let escaped_value = Eliom_lib.escaped_value
+
+  let client_value closure_id args =
+    let instance_id = Eliom_lib.fresh_ix () in
+    register_client_value_data ~closure_id ~instance_id ~args:(to_poly args);
+    create_client_value
+      (Client_value_server_repr.create closure_id instance_id)
+
+  let close_server_section compilation_unit_id =
+    close_server_section ~compilation_unit_id
+
+  let close_client_section compilation_unit_id =
+    close_client_section ~compilation_unit_id
+
+  let set_global b = is_global := b
+
+end
