@@ -22,7 +22,7 @@
 (** Creation and manipulation of Eliom services.
 
     See the Eliom manual for a detailed introduction to the concept of
-    {% <<a_manual chapter="services"|Eliom services>>%}.
+    {% <<a_manual chapter="server-services"|Eliom services>>%}.
 
     {% <<outline>> %}
  *)
@@ -124,16 +124,15 @@ type registrable = [ `Registrable | `Unregistrable ]
     - [ 'c] describes the services's kind: attached or non-attached,
             internal or external, GET only or with POST
             parameters. It is a subtype of {!service_kind}.
-    - [ 'd] is a phantom type, subtype of {!suff} stating the kind
-            of parameters it uses: suffix or not.
-    - [ 'e] is the type of GET parameters names. See {!Eliom_parameter.param_name} and
+    - the type paremeter of subtype {!suff} states the kind
+      of parameters it uses: suffix or not.
+    - [ 'd] is the type of GET parameters names. See {!Eliom_parameter.param_name} and
             form generation functions (e. g. {!Eliom_registration.Html5.get_form}).
-    - [ 'f] is the type of POST parameters names. See {!Eliom_parameter.param_name} and
+    - [ 'e] is the type of POST parameters names. See {!Eliom_parameter.param_name} and
             form generation functions (e. g. {!Eliom_registration.Html5.post_form}).
-    - [ 'g] is a phantom type,  subtype of {!registrable},
-            telling if it is possible to register a handler
-            on this service.
-    - [ 'h] is an information on what the service returns.
+    - the type parameter of subtype {!registrable} tells if it is possible to
+      register a handler on this service.
+    - [ 'f] is an information on what the service returns.
             See {!Eliom_registration.kind}.
 *)
 type ('a,'b,+'c,+'d,+'e,+'f,+'g,+'h) service
@@ -195,7 +194,7 @@ val register_eliom_module : string -> (unit -> unit) -> unit
     service will keep persistent (resp. all) non localized GET
     parameters of the current service. The default is [`None]. See the
     eliom manual for more information about {% <<a_manual
-    chapter="params" fragment="nonlocalizedparameters"|non localized
+    chapter="server-params" fragment="nonlocalizedparameters"|non localized
     parameters>>%}.
 *)
 val service :
@@ -291,7 +290,7 @@ val post_service :
 (** {3 Attached coservices} *)
 
 (** The function [coservice ~fallback ~get_params] creates an {%
-    <<a_manual chapter="service" fragment="coservices"|attached
+    <<a_manual chapter="server-services" fragment="attached_coservices"|attached
     coservice>>%} at the same path than the service [fallback] and
     taking [get_params] as GET parameters.
 
@@ -313,7 +312,7 @@ val post_service :
     number of times. The default is "no limitation".
 
     If the optional [~csrf_safe] parameter is [true], it will create a
-    {% <<a_manual chapter="security" fragment="csrf"|"CSRF-safe"
+    {% <<a_manual chapter="server-security" fragment="csrf"|"CSRF-safe"
     service>>%}. In that case the [~name] parameter is ignored. The
     default is [false].
 
@@ -352,7 +351,7 @@ val coservice :
    [> `Registrable ], 'return2) service
 
 (** The function [post_coservice ~fallback ~get_params] creates an {%
-    <<a_manual chapter="service" fragment="coservices"|attached
+    <<a_manual chapter="server-services" fragment="attached_coservices"|attached
     coservice>>%} with the same path and GET parameters than the
     service [fallback] and taking [post_params] as POST
     parameters.
@@ -388,7 +387,7 @@ val post_coservice :
 (** {3 Non attached coservices} *)
 
 (** The function [coservice' ~get_param] creates a {% <<a_manual
-    chapter="services" fragment="non-attached_coservices"|non-attached
+    chapter="server-services" fragment="non-attached_coservices"|non-attached
     coservice>>%} taking [get_params] as extra GET parameters.
 
     GET parameters of [coservice'] couldn't contain a suffix
@@ -415,7 +414,7 @@ val coservice' :
 
 
 (** The function [post_coservice' ~get_param] creates a {% <<a_manual
-    chapter="services" fragment="non-attached_coservices"|non-attached
+    chapter="server-services" fragment="non-attached_coservices"|non-attached
     coservice>>%} taking [post_params] as POST parameters.
 
     POST parameters couldn't contain a suffix parameter.
@@ -556,7 +555,7 @@ val preapply :
 (** The function [add_non_localized_get_parameters ~params ~service]
     Adds non localized GET parameters [params] to [service]. See the
     Eliom manual for more information about {% <<a_manual
-    chapter="params" fragment="nonlocalizedparameters"|non localized
+    chapter="server-params" fragment="nonlocalizedparameters"|non localized
     parameters>>%}. *)
 val add_non_localized_get_parameters :
   params:('p, [ `WithoutSuffix ], 'pn) non_localized_params ->
@@ -585,38 +584,6 @@ val unregister :
   unit
 
 
-(** {2 Eliom application services} *)
-
-(** The function [onload evt] will register the function [evt] to be
-    executed on client side once the [App] page generated by
-    the current service handler is loaded. This function should not be
-    called outside of a service handler.
-
-    Use it with Eliom's syntax extension for client side code, for
-    example:
-
-    [onload {{ Firebug.console.debug(Js.string "Hello world!") }}].
-*)
-val onload : Dom_html.event Xml.caml_event_handler -> unit
-
-(** The function [onunload evt] will register the function [evt] to be
-    executed on client side when leaving the [App] page
-    generated by the current service handler. This function should not
-    be called outside of a service handler.
-
-    Use it with Eliom's syntax extension for client side code, for
-    example:
-
-    [onunload {{ Firebug.console.debug(Js.string "Goodbye!") }}].*)
-val onunload : Dom_html.event Xml.caml_event_handler -> unit
-
-
-
-
-
-
-
-
 (**/**)
 val get_get_or_post :
   ('a, 'b,
@@ -714,9 +681,6 @@ val get_send_appl_content : ('a, 'b, 'c, 'd, 'e, 'f, 'g, 'h) service -> send_app
 val xhr_with_cookies :
   ('a, 'b, 'c, 'd, 'e, 'f, 'g, 'h) service -> string option option
 
-val get_onload : Eliom_common.server_params -> Dom_html.event Xml.caml_event_handler list
-val get_onunload : Eliom_common.server_params -> Dom_html.event Xml.caml_event_handler list
-
 val pre_wrap :
   ('a, 'b, 'c, 'd, 'e, 'f, 'g, 'rr) service ->
   ('a, 'b, 'c, 'd, 'e, 'f, 'g, 'rr) service
@@ -724,3 +688,49 @@ val pre_wrap :
 val eliom_appl_answer_content_type : string
 
 exception Wrong_session_table_for_CSRF_safe_coservice
+
+val get_global_data : unit -> poly Eliom_lib_base.global_data
+val get_request_data : unit -> request_data
+
+module Syntax_helpers : sig
+
+  (** Registers a client value datum for the next server section when
+      executed in a global_data
+      (cf. {!Eliom_service.Syntax_helpers.set_global}) or in the
+      request_data when executed in a request. *)
+  val client_value : int64 -> 'args -> 'a client_value
+
+  (** All client values created between [set_global true] and
+      [set_global false] are considered global client values
+      (cf. <<a_manual chapter="clientserver-language" chapter="clientvalues"|the
+      manual>>).  *)
+  val set_global : bool -> unit
+
+  (** Called at the end of each server or shared section. The argument
+      identifies the compilation unit.
+
+      Adds the list of recently registered
+      {!Eliom_lib_base.client_value_datum}s into the queue of server
+      section data of the compilation unit
+      ({!Eliom_lib_base.compilation_unit_global_data}).
+
+      Called in parallel with <<a_api
+      subproject="client"|Eliom_client.Syntax_helpers.close_server_section>>.  *)
+  val close_server_section : string -> unit
+
+  (** Called at the end of every client or shared section. The first
+      argument identifies the compilation unit. The second is the list
+      of novel injections in that section.
+
+      Adds a list of {!Eliom_lib_base.injection_datum}s into the queue
+      of client section data of the compilation unit
+      ({!Eliom_lib_base.compilation_unit_global_data}).
+
+      Called in parallel with <<a_api
+      subproject="client"|Eliom_client.Syntax_helpers.open_client_section>>.  *)
+  val close_client_section : string -> (string * (unit -> poly)) list -> unit
+
+  (** Convert any value to a {! Eliom_lib.escaped_value} for usage in
+      the [args] argument to {! Eliom_service.Syntax_helpers.client_value}. *)
+  val escaped_value : 'a -> escaped_value
+end
