@@ -34,7 +34,8 @@ open Ocsigen_extensions
 
 (** {3 Closing sessions, removing state data and services} *)
 
-(** Delete server-side state data and services for a session,
+(** Delete server-side (vatile and persistent) state data and services
+    for a session,
     a group of sessions, a client process or a request.
 
     Use that function to close a session (using scope [Eliom_common.session]).
@@ -53,6 +54,8 @@ val discard :
   unit ->
   unit Lwt.t
 
+(* Discard services and (volatile and persistent) data
+   for all user and request scopes *)
 val discard_all_scopes :
   ?secure:bool ->
   unit ->
@@ -136,14 +139,14 @@ val persistent_data_state_status :
 
 (** Ask the browser to record a cookie. *)
 val set_cookie :
-  ?cookie_scope:Eliom_common.cookie_scope ->
+  ?cookie_level:Eliom_common.cookie_level ->
   ?path:string list ->
   ?exp:float ->
   ?secure:bool -> name:string -> value:string -> unit -> unit
 
 (** Ask the browser to remove a cookie. *)
 val unset_cookie :
-  ?cookie_scope:Eliom_common.cookie_scope ->
+  ?cookie_level:Eliom_common.cookie_level ->
   ?path:string list ->
   name:string -> unit -> unit
 
@@ -183,7 +186,8 @@ val set_service_session_group :
   string ->
   unit
 
-(** Remove the session from its group *)
+(** Remove the session from its group.
+    Will not close the session if it contains data. *)
 val unset_service_session_group :
   ?set_max: int ->
   ?scope:Eliom_common.session_scope ->
@@ -223,7 +227,8 @@ val set_volatile_data_session_group :
   string ->
   unit
 
-(** Remove the session from its group *)
+(** Remove the session from its group.
+    Will not close the session if it contains data. *)
 val unset_volatile_data_session_group :
   ?set_max: int ->
   ?scope:Eliom_common.session_scope ->
@@ -264,7 +269,8 @@ val set_persistent_data_session_group :
   string ->
   unit Lwt.t
 
-(** Remove the session from its group. *)
+(** Remove the session from its group.
+    Will not close the session if it contains data. *)
 val unset_persistent_data_session_group :
   ?scope:Eliom_common.session_scope ->
   ?secure:bool ->
@@ -430,7 +436,7 @@ val set_max_volatile_states_for_group_or_subnet :
     for an infinite time on browsers.
 
     By default, it will affect regular browser cookies (sessions).
-    But if you set [~cookie_scope:`Client_process],
+    But if you set [~cookie_level:`Client_process],
     it will only affect the client-side Eliom process (if there is one),
     which simulates some kind of "tab cookies".
 *)
@@ -439,7 +445,7 @@ val set_max_volatile_states_for_group_or_subnet :
     (see above).
 *)
 val set_service_cookie_exp_date :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:Eliom_common.cookie_scope ->
   ?secure:bool ->
   float option ->
   unit
@@ -448,7 +454,7 @@ val set_service_cookie_exp_date :
     above).
 *)
 val set_volatile_data_cookie_exp_date :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:Eliom_common.cookie_scope ->
   ?secure:bool ->
   float option ->
   unit
@@ -457,7 +463,7 @@ val set_volatile_data_cookie_exp_date :
     above).
 *)
 val set_persistent_data_cookie_exp_date :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:Eliom_common.cookie_scope ->
   ?secure:bool ->
   float option ->
   unit Lwt.t
@@ -480,9 +486,10 @@ val set_persistent_data_cookie_exp_date :
     functions like
     {!Eliom_state.set_volatile_data_state_timeout} won't be affected.
 
-    If [~scope_name] is not present, it is the default for all scope names,
-    and in that case [recompute_expdates] is ignored. [~scope_name:None]
-    means the default scope name.
+    If [~scope_hierarchy] is not present,
+    it is the default for all scope hierarchies,
+    and in that case [recompute_expdates] is ignored. [~scope_hierarchy:None]
+    means the default scope hierarchy.
 
     If [~override_configfile] is [true] (default ([false]),
     then the function will set the timeout even if it has been
@@ -500,52 +507,56 @@ val set_persistent_data_cookie_exp_date :
     service session and volatile data session).
 *)
 val set_global_volatile_state_timeout :
-  scope:[< Eliom_common.user_scope ] ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
+  ?secure: bool ->
   ?recompute_expdates:bool ->
   ?override_configfile:bool ->
   float option -> unit
 
 val set_default_global_service_state_timeout :
-  cookie_scope:[< Eliom_common.cookie_scope ] ->
+  cookie_level:[< Eliom_common.cookie_level ] ->
   ?override_configfile:bool ->
   float option -> unit
 
 (** Sets the (server side) timeout for service states.
 *)
 val set_global_service_state_timeout :
-  scope:[< Eliom_common.user_scope ] ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
+  ?secure: bool ->
   ?recompute_expdates:bool ->
   ?override_configfile:bool ->
   float option -> unit
 
 val set_default_global_service_state_timeout :
-  cookie_scope:[< Eliom_common.cookie_scope ] ->
+  cookie_level:[< Eliom_common.cookie_level ] ->
   ?override_configfile:bool ->
   float option -> unit
 
 (** Sets the (server side) timeout for volatile (= "in memory") data states.
 *)
 val set_global_volatile_data_state_timeout :
-  scope:[< Eliom_common.user_scope ] ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
+  ?secure: bool ->
   ?recompute_expdates:bool ->
   ?override_configfile:bool ->
   float option -> unit
 
 val set_default_global_volatile_data_state_timeout :
-  cookie_scope:[< Eliom_common.cookie_scope ] ->
+  cookie_level:[< Eliom_common.cookie_level ] ->
   ?override_configfile:bool ->
   float option -> unit
 
 (** Sets the (server side) timeout for persistent states.
 *)
 val set_global_persistent_data_state_timeout :
-  scope:[< Eliom_common.user_scope ] ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
+  ?secure: bool ->
   ?recompute_expdates:bool ->
   ?override_configfile:bool ->
   float option -> unit
 
 val set_default_global_persistent_data_state_timeout :
-  cookie_scope:[< Eliom_common.cookie_scope ] ->
+  cookie_level:[< Eliom_common.cookie_level ] ->
   ?override_configfile:bool ->
   float option -> unit
 
@@ -553,19 +564,22 @@ val set_default_global_persistent_data_state_timeout :
 (** Returns the (server side) timeout for service states.
 *)
 val get_global_service_state_timeout :
-  scope:[< Eliom_common.user_scope ] ->
+  ?secure: bool ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   unit -> float option
 
 (** Returns the (server side) timeout for "volatile data" states.
 *)
 val get_global_volatile_data_state_timeout :
-  scope:[< Eliom_common.user_scope ] ->
+  ?secure: bool ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   unit -> float option
 
 (** Returns the (server side) timeout for persistent states.
 *)
 val get_global_persistent_data_state_timeout :
-  scope:[< Eliom_common.user_scope ] ->
+  ?secure: bool ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   unit -> float option
 
 
@@ -575,14 +589,14 @@ val get_global_persistent_data_state_timeout :
 (** sets the timeout for service state (server side) for current user,
    in seconds. [None] = no timeout *)
 val set_service_state_timeout :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:Eliom_common.cookie_scope ->
   ?secure:bool ->
   float option -> unit
 
 (** remove the service state timeout for current user
    (and turn back to the default). *)
 val unset_service_state_timeout :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   ?secure:bool ->
   unit -> unit
 
@@ -590,7 +604,7 @@ val unset_service_state_timeout :
     [None] = no timeout
  *)
 val get_service_state_timeout :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   ?secure:bool ->
   unit -> float option
 
@@ -598,14 +612,14 @@ val get_service_state_timeout :
 (** sets the (server side) timeout for volatile data state for current user,
    in seconds. [None] = no timeout *)
 val set_volatile_data_state_timeout :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   ?secure:bool ->
   float option -> unit
 
 (** remove the "volatile data" state timeout for current user
    (and turn back to the default). *)
 val unset_volatile_data_state_timeout :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   ?secure:bool ->
   unit -> unit
 
@@ -613,7 +627,7 @@ val unset_volatile_data_state_timeout :
     [None] = no timeout
  *)
 val get_volatile_data_state_timeout :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   ?secure:bool ->
   unit -> float option
 
@@ -622,21 +636,21 @@ val get_volatile_data_state_timeout :
 (** sets the (server side) timeout for persistent state for current user,
    in seconds. [None] = no timeout *)
 val set_persistent_data_state_timeout :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   ?secure:bool ->
   float option -> unit Lwt.t
 
 (** remove the persistent state timeout for current user
    (and turn back to the default). *)
 val unset_persistent_data_state_timeout :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   ?secure:bool ->
   unit -> unit Lwt.t
 
 (** returns the persistent state timeout for current user.
     [None] = no timeout *)
 val get_persistent_data_state_timeout :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   ?secure:bool ->
   unit -> float option Lwt.t
 
@@ -668,6 +682,7 @@ val discard_everything : unit -> unit Lwt.t
 (** Discard all services and persistent and volatile data for one scope. *)
 val discard_all :
   scope:Eliom_common.user_scope ->
+  ?secure:bool ->
   unit ->
   unit Lwt.t
 (*VVV missing: scope group *)
@@ -681,6 +696,7 @@ val discard_all :
 val discard_all_data :
   ?persistent:bool ->
   scope:Eliom_common.user_scope ->
+  ?secure:bool ->
   unit ->
   unit Lwt.t
 (*VVV missing: scope group *)
@@ -690,13 +706,14 @@ val discard_all_data :
 (** Remove all services registered for clients for the given scope. *)
 val discard_all_services :
   scope:Eliom_common.user_scope ->
+  ?secure:bool ->
   unit ->
   unit Lwt.t
 (*VVV missing: scope group *)
 (*VVV missing ~secure? *)
 
 
-module Session_admin : sig
+module Ext : sig
 
   (** Type used to describe session timeouts *)
   type timeout =
@@ -704,124 +721,221 @@ module Session_admin : sig
     | TNone   (** explicitly set no timeout *)
     | TSome of float (** timeout duration in seconds *)
 
-  type service_session
-  type data_session
-  type persistent_session
+  (** These types are used to get or set information about browser
+      or process cookies (like timeouts). *)
 
-  val close_service_session :
-    ?close_group:bool ->
-    session:service_session ->
+  type service_cookie_info
+  type data_cookie_info
+  type persistent_cookie_info
+
+  (** The type of states. The first parameter corresponds to the scope level
+      and the second one to the kind of state (volatile or persistent data,
+      or service state) *)
+  type (+'a, +'b) state
+
+  (** [volatile_data_group_state ~scope n] returns the state corresponding to
+      the group named [n] in scope [scope]. *)
+  val volatile_data_group_state :
+    ?scope:Eliom_common.session_group_scope -> string ->
+    ([> `Session_group ], [> `Data ]) state
+
+  (** Same for persistent data *)
+  val persistent_data_group_state :
+    ?scope:Eliom_common.session_group_scope -> string ->
+    ([> `Session_group ], [> `Pers ]) state
+
+  (** Same for services *)
+  val service_group_state :
+    ?scope:Eliom_common.session_group_scope -> string ->
+    ([> `Session_group ], [> `Service ]) state
+
+  (** [current_volatile_session_state ~scope] returns the state corresponding
+      to current session in scope [scope]. *)
+  val current_volatile_session_state :
+    ?secure:bool ->
+    ?scope:Eliom_common.session_scope ->
+    unit ->
+    ([< `Session ], [< `Data ]) state
+
+  (** Same for persistent data *)
+  val current_persistent_session_state :
+    ?secure:bool ->
+    ?scope:Eliom_common.session_scope ->
+    unit ->
+    ([< `Session ], [< `Pers ]) state Lwt.t
+
+  (** Same for services *)
+  val current_service_session_state :
+    ?secure:bool ->
+    ?scope:Eliom_common.session_scope ->
+    unit ->
+    ([< `Session ], [< `Service ]) state
+
+  (** Discard external states *)
+  val discard_state : state : ('a, 'b) state -> unit Lwt.t
+
+  (** Fold all sessions in a groups, or all client processes in a session. *)
+  val fold_volatile_sub_states :
+    state : ([< `Session_group | `Session ],
+             [< `Data | `Service ] as 'k) state ->
+    ('a -> ([< `Session | `Client_process ], 'k) state -> 'a) ->
+    'a -> 'a
+
+  (** Iter on all sessions in a groups, or all client processes in a session. *)
+  val iter_volatile_sub_states :
+    state: ([< `Session_group | `Session ],
+            [< `Data | `Service ] as 'k) state ->
+    (([< `Session | `Client_process ], 'k) state -> unit) ->
     unit
 
-  val close_volatile_data_session :
-    ?close_group:bool ->
-    session:data_session ->
-    unit
+  (** Fold all sessions in a groups, or all client processes in a session. *)
+  val fold_sub_states :
+    state : ([< `Session_group | `Session ],
+             [< `Data | `Pers | `Service ] as 'k) state ->
+    ('a -> ([< `Session | `Client_process ], 'k) state -> 'a Lwt.t) ->
+    'a -> 'a Lwt.t
 
-  val close_persistent_data_session :
-    ?close_group:bool ->
-    session:persistent_session ->
+  (** Iter on all sessions in a groups, or all client processes in a session. *)
+  val iter_sub_states :
+    state: ([< `Session_group | `Session ], 'k) state ->
+    (([< `Session | `Client_process ], 'k) state -> unit Lwt.t) ->
     unit Lwt.t
 
-  (** Raises [Not_found] if no data in the table for the session. *)
-  val get_volatile_session_data :
-    session:data_session ->
-    table:'a volatile_table ->
-    'a
+  module Low_level : sig
+    (** Functions to access table data.
+        Prefer using Eliom references. *)
 
-  (** Fails with lwt exception [Not_found]
-      if no data in the table for the session. *)
-  val get_persistent_session_data :
-    session:persistent_session ->
-    table:'a persistent_table ->
-    'a Lwt.t
+    (** Raises [Not_found] if no data in the table for the cookie. *)
+    val get_volatile_data :
+      state:([< `Session_group | `Session | `Client_process ],
+             [< `Data ]) state ->
+      table:'a volatile_table ->
+      'a
 
-  val remove_volatile_session_data :
-      session:data_session -> table:'a volatile_table -> unit
-  val remove_persistent_session_data :
-      session:persistent_session -> table:'a persistent_table -> unit Lwt.t
+    (** Fails with lwt exception [Not_found]
+        if no data in the table for the cookie. *)
+    val get_persistent_data :
+      state:([< `Session_group | `Session | `Client_process ],
+             [< `Pers ]) state ->
+      table:'a persistent_table ->
+      'a Lwt.t
 
-  val get_service_scope_name :
-    session:service_session -> Eliom_common.scope_name
+    val set_volatile_data :
+      state:([< `Session_group | `Session | `Client_process ],
+             [< `Data ]) state ->
+      table:'a volatile_table ->
+      'a -> unit
 
-  val get_volatile_data_scope_name : session:data_session ->
-    Eliom_common.scope_name
+    (** Fails with lwt exception [Not_found]
+        if no data in the table for the cookie. *)
+    val set_persistent_data :
+      state:([< `Session_group | `Session | `Client_process ],
+             [< `Pers ]) state ->
+      table:'a persistent_table ->
+      'a -> unit Lwt.t
 
-  val get_persistent_data_scope_name :
-      session:persistent_session -> Eliom_common.scope_name
+    val remove_volatile_data :
+      state:([< `Session_group | `Session | `Client_process ],
+             [< `Data ]) state ->
+      table:'a volatile_table -> unit
 
-  val get_service_session_cookie_scope :
-    session:service_session -> Eliom_common.cookie_scope
-  val get_volatile_data_session_cookie_scope : session:data_session ->
-    Eliom_common.cookie_scope
-  val get_persistent_data_session_cookie_scope :
-    session:persistent_session -> Eliom_common.cookie_scope
+    val remove_persistent_data :
+      state:([< `Session_group | `Session | `Client_process ],
+             [< `Pers ]) state ->
+      table:'a persistent_table -> unit Lwt.t
 
-  val set_service_session_timeout :
-      session:service_session -> float option -> unit
-  val set_volatile_data_session_timeout :
-      session:data_session -> float option -> unit
-  val set_persistent_data_session_timeout :
-      session:persistent_session -> float option -> unit Lwt.t
+  end
 
-  val get_service_session_timeout :
-      session:service_session -> timeout
+  (** Get the infomration about cookies (timeouts, etc.) *)
+  val get_service_cookie_info :
+    ([< Eliom_common.cookie_level ], [ `Service ]) state -> service_cookie_info
 
-  val get_volatile_data_session_timeout :
-      session:data_session -> timeout
+  val get_volatile_data_cookie_info :
+    ([< Eliom_common.cookie_level ], [ `Data ]) state -> data_cookie_info
 
-  val get_persistent_data_session_timeout :
-      session:persistent_session -> timeout
+  val get_persistent_cookie_info :
+    ([< Eliom_common.cookie_level ], [ `Pers ]) state ->
+    persistent_cookie_info Lwt.t
 
-  val unset_service_session_timeout :
-      session:service_session -> unit
-  val unset_volatile_data_session_timeout :
-      session:data_session -> unit
-  val unset_persistent_data_session_timeout :
-      session:persistent_session -> unit Lwt.t
+  val get_service_cookie_scope :
+    cookie:service_cookie_info -> Eliom_common.user_scope
+  val get_volatile_data_cookie_scope : cookie:data_cookie_info ->
+    Eliom_common.user_scope
+  val get_persistent_data_cookie_scope :
+    cookie:persistent_cookie_info -> Eliom_common.user_scope
 
-  (** Iterator on service sessions. [Lwt_unix.yield] is called automatically
-     after each iteration.
+  val set_service_cookie_timeout :
+    cookie:service_cookie_info -> float option -> unit
+  val set_volatile_data_cookie_timeout :
+    cookie:data_cookie_info -> float option -> unit
+  val set_persistent_data_cookie_timeout :
+    cookie:persistent_cookie_info -> float option -> unit Lwt.t
+
+  val get_service_cookie_timeout :
+    cookie:service_cookie_info -> timeout
+
+  val get_volatile_data_cookie_timeout :
+    cookie:data_cookie_info -> timeout
+
+  val get_persistent_data_cookie_timeout :
+    cookie:persistent_cookie_info -> timeout
+
+  val unset_service_cookie_timeout :
+    cookie:service_cookie_info -> unit
+  val unset_volatile_data_cookie_timeout :
+    cookie:data_cookie_info -> unit
+  val unset_persistent_data_cookie_timeout :
+    cookie:persistent_cookie_info -> unit Lwt.t
+
+  (** Returns a list containing the names of all session group
+      that are available for this site. *)
+  val get_session_group_list : unit -> string list
+
+  (** Iterator on all active service cookies.
+      [Lwt_unix.yield] is called automatically after each iteration.
    *)
-  val iter_service_sessions :
-    (service_session -> unit Lwt.t) -> unit Lwt.t
+  val iter_service_cookies :
+    (service_cookie_info -> unit Lwt.t) -> unit Lwt.t
 
-  (** Iterator on data sessions. [Lwt_unix.yield] is called automatically
+  (** Iterator on data cookies. [Lwt_unix.yield] is called automatically
       after each iteration.
    *)
-  val iter_volatile_data_sessions :
-    (data_session -> unit Lwt.t) -> unit Lwt.t
+  val iter_volatile_data_cookies :
+    (data_cookie_info -> unit Lwt.t) -> unit Lwt.t
 
-  (** Iterator on persistent sessions. [Lwt_unix.yield] is called automatically
+  (** Iterator on persistent cookies. [Lwt_unix.yield] is called automatically
       after each iteration. *)
-  val iter_persistent_data_sessions :
-    (persistent_session -> unit Lwt.t) -> unit Lwt.t
+  val iter_persistent_data_cookies :
+    (persistent_cookie_info -> unit Lwt.t) -> unit Lwt.t
 
-  (** Iterator on service sessions. [Lwt_unix.yield] is called automatically
+  (** Iterator on service cookies. [Lwt_unix.yield] is called automatically
       after each iteration.
   *)
-  val fold_service_sessions :
-    (service_session -> 'b -> 'b Lwt.t) -> 'b -> 'b Lwt.t
+  val fold_service_cookies :
+    (service_cookie_info -> 'b -> 'b Lwt.t) -> 'b -> 'b Lwt.t
 
-  (** Iterator on data sessions. [Lwt_unix.yield] is called automatically
+  (** Iterator on data cookies. [Lwt_unix.yield] is called automatically
      after each iteration.
    *)
-  val fold_volatile_data_sessions :
-    (data_session -> 'b -> 'b Lwt.t) -> 'b  -> 'b Lwt.t
+  val fold_volatile_data_cookies :
+    (data_cookie_info -> 'b -> 'b Lwt.t) -> 'b  -> 'b Lwt.t
 
-  (** Iterator on persistent sessions. [Lwt_unix.yield] is called automatically
+  (** Iterator on persistent cookies. [Lwt_unix.yield] is called automatically
      after each iteration. *)
-  val fold_persistent_data_sessions :
-    (persistent_session -> 'b -> 'b Lwt.t) -> 'b -> 'b Lwt.t
+  val fold_persistent_data_cookies :
+    (persistent_cookie_info -> 'b -> 'b Lwt.t) -> 'b -> 'b Lwt.t
+
+(**/**)
+  val untype_state : ('a, 'b) state -> ('c, 'd) state
 
 end
 
 
-
 (*****************************************************************************)
+(**/**)
 (** {3 Session data (deprecated interface)} *)
 
-(** This interface is deprecated. Now use Eliom references. *)
+(** This is the low level interface (deprecated). Use now Eliom references. *)
 
 (** The type used for getting data from a state. *)
 type 'a state_data =
@@ -832,7 +946,7 @@ type 'a state_data =
 (** {4 In memory state data} *)
 
 (** creates a table in memory where you can store the session data for
-    all users. (deprecated)
+    all users. (low level)
 
     {e Warning: This functions must be called when the site
     information is available, that is, either
@@ -849,13 +963,13 @@ val create_volatile_table :
   ?secure:bool ->
   unit -> 'a volatile_table
 
-(** gets session data for the current session (if any).  (deprecated) *)
+(** gets session data for the current session (if any).  (low level) *)
 val get_volatile_data :
   table:'a volatile_table ->
   unit ->
   'a state_data
 
-(** sets session data for the current session.  (deprecated) *)
+(** sets session data for the current session.  (low level) *)
 val set_volatile_data :
   table:'a volatile_table ->
   'a ->
@@ -864,33 +978,33 @@ val set_volatile_data :
 (** removes session data for the current session
     (but does not close the session).
     If the session does not exist, does nothing.
-    (deprecated)
+    (low level)
  *)
 val remove_volatile_data :
   table:'a volatile_table ->
   unit ->
   unit
+(**/**)
 
 
+(**/**)
 (** {4 Persistent state data} *)
-
-
 (** creates a table on hard disk where you can store the session data for
-    all users. It uses {!Ocsipersist}.  (deprecated) *)
+    all users. It uses {!Ocsipersist}.  (low level) *)
 val create_persistent_table :
   scope:Eliom_common.user_scope ->
   ?secure:bool ->
   string -> 'a persistent_table
 
 (** gets persistent session data for the current persistent session (if any).
-    (deprecated) *)
+    (low level) *)
 val get_persistent_data :
   table:'a persistent_table ->
   unit ->
   'a state_data Lwt.t
 
 (** sets persistent session data for the current persistent session.
-    (deprecated) *)
+    (low level) *)
 val set_persistent_data :
   table:'a persistent_table ->
   'a ->
@@ -899,20 +1013,15 @@ val set_persistent_data :
 (** removes session data for the current persistent session
     (but does not close the session).
     If the session does not exist, does nothing.
-    (deprecated)
+    (low level)
  *)
 val remove_persistent_data :
   table:'a persistent_table ->
   unit ->
   unit Lwt.t
-
-
-
-
-
-
-
 (**/**)
+
+
 (*
 (** {3 Default timeouts} *)
 
@@ -974,20 +1083,19 @@ val set_default_volatile_data_session_timeout : float option -> unit
     *)
 val set_default_persistent_data_session_timeout : float option -> unit
 *)
-(**/**)
 
 
 (*****************************************************************************)
-(** {3 Other low level functions} *)
-
-(** You probably don't need these functions. *)
 
+(**/**)
+(** {3 Other low level functions}
+    You probably don't need these functions. *)
 
 (** returns the value of the Eliom's cookies for one persistent session.
     Returns [None] is no session is active.
  *)
 val get_persistent_data_cookie :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:Eliom_common.cookie_scope ->
   ?secure:bool ->
   unit -> string option Lwt.t
 
@@ -995,7 +1103,7 @@ val get_persistent_data_cookie :
     Returns [None] is no session is active.
  *)
 val get_service_cookie :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:Eliom_common.cookie_scope ->
   ?secure:bool ->
   unit -> string option
 
@@ -1003,9 +1111,10 @@ val get_service_cookie :
     Returns [None] is no session is active.
  *)
 val get_volatile_data_cookie :
-  scope:Eliom_common.user_scope ->
+  cookie_scope:Eliom_common.cookie_scope ->
   ?secure:bool ->
   unit -> string option
+(**/**)
 
 
 
@@ -1015,15 +1124,15 @@ val get_volatile_data_cookie :
 
 (**/**)
 (*****************************************************************************)
-val number_of_service_sessions : unit -> int
+val number_of_service_cookies : unit -> int
 
-val number_of_volatile_data_sessions : unit -> int
+val number_of_volatile_data_cookies : unit -> int
 
 val number_of_tables : unit -> int
 
 val number_of_table_elements : unit -> int list
 
-val number_of_persistent_data_sessions : unit -> int Lwt.t
+val number_of_persistent_data_cookies : unit -> int Lwt.t
 
 val number_of_persistent_tables : unit -> int
 
@@ -1053,4 +1162,3 @@ val create_volatile_table_during_session_ :
   secure:bool ->
   Eliom_common.sitedata ->
   'a volatile_table
-
