@@ -21,7 +21,7 @@
 (*****************************************************************************)
 (** Internal functions used by Eliom:                                        *)
 (** Tables of services (global and session tables,                           *)
-(** persistant and volatile data tables)                                     *)
+(** persistent and volatile data tables)                                     *)
 (** Store and load services                                                  *)
 (*****************************************************************************)
 (*****************************************************************************)
@@ -118,7 +118,7 @@ let new_sitedata =
               Eliom_common.empty_tables
                 !default_max_anonymous_services_per_subnet
                 false;
-	   registered_scope_names = String.Set.empty;
+	   registered_scope_hierarchies = Eliom_common.Hier_set.empty;
            session_services = Eliommod_cookies.new_service_cookie_table ();
            session_data = Eliommod_cookies.new_data_cookie_table ();
            group_of_groups = gog;
@@ -232,10 +232,10 @@ let parse_eliom_option
     let rec aux ((v, sn, ct) as res) = function
       | [] -> res
       | ("value", s)::l -> aux (Some s, sn, ct) l
-      | ("sessionname", sn)::l -> aux (v, Some sn, ct) l
-      | ("sessiontype", "browser")::l -> aux (v, sn, `Session) l
-      | ("sessiontype", "tab")::l -> aux (v, sn, `Client_process) l
-      | ("sessiontype", _)::l -> 
+      | ("hierarchyname", sn)::l -> aux (v, Some sn, ct) l
+      | ("level", "browser")::l -> aux (v, sn, `Session) l
+      | ("level", "tab")::l -> aux (v, sn, `Client_process) l
+      | ("level", _)::l -> 
           raise 
             (Error_in_config_file
                ("Eliom: Wrong attribute value for sessiontype in "^tn^" tag"))
@@ -645,6 +645,7 @@ let parse_config hostpattern conf_info site_dir =
 (*--- (mutatis mutandis for the following line:) *)
   Eliom_common.absolute_change_sitedata sitedata;
   let firsteliomtag = ref true in
+  let eliommodulewarningdisplayed = ref false in
   let rec parse_default_links_xhr atts default_links_xhr = function
     | [] -> default_links_xhr, List.rev atts
     | ("xhr-links", str_value)::suite ->
@@ -698,12 +699,10 @@ let parse_config hostpattern conf_info site_dir =
           load_eliom_module sitedata file_or_name content;
           exception_during_eliommodule_loading := false
         | _ -> ());
-      if Eliom_extension.get_eliom_extension ()
-        != default_module_action
-      then
-        Eliommod_pagegen.gen
-          (Some (Eliom_extension.get_eliom_extension ()))
-          sitedata
+      if Eliom_extension.get_eliom_extension () != default_module_action
+      then Eliommod_pagegen.gen
+        (Some (Eliom_extension.get_eliom_extension ()))
+        sitedata
       else gen_nothing ()
     | Element ("eliom", atts, content) ->
 (*--- if we put the line "new_sitedata" here, then there is
@@ -713,32 +712,35 @@ let parse_config hostpattern conf_info site_dir =
   browsers manage cookies (one cookie for one site).
   Thus we can have one site in several cmo (with one session).
  *)
-        let set_timeout (f : ?fullsessname:Eliom_common.fullsessionname ->
-                         ?cookie_scope:[< Eliom_common.cookie_scope ] ->
+        let set_timeout (f : ?full_st_name:Eliom_common.full_state_name ->
+                         ?cookie_level:[< Eliom_common.cookie_level ] ->
                          recompute_expdates:bool ->
                          bool -> bool -> Eliom_common.sitedata ->
                          float option -> unit)
-            cookie_type state_name_oo v =
-	  let make_fullsessname state_name =
-	    let state_name : Eliom_common.scope_name =
-	      match state_name with
-		| None -> `Default_ref_name
-		| Some s when String.lowercase s = "default" -> `Default_ref_name
-		| Some s when String.lowercase s = "comet" -> `Default_comet_name
-		| Some s -> `String s
+            cookie_type state_hier_oo v =
+	  let make_full_st_name state_hier =
+	    let state_hier : Eliom_common.scope_hierarchy =
+	      match state_hier with
+		| None -> Eliom_common_base.Default_ref_hier
+		| Some s when String.lowercase s = "default" ->
+                  Eliom_common_base.Default_ref_hier
+		| Some s when String.lowercase s = "comet" ->
+                  Eliom_common_base.Default_comet_hier
+		| Some s -> Eliom_common_base.User_hier s
 	    in
 	    let scope =
 	      match cookie_type with
-		| `Session -> `Session state_name
-		| `Client_process -> `Client_process state_name
+		| `Session -> `Session state_hier
+		| `Client_process -> `Client_process state_hier
 	    in
-	    Eliom_common.make_fullsessname2
+	    Eliom_common.make_full_state_name2
               sitedata.Eliom_common.site_dir_string
-              scope
+              false (*VVV and secure??? *)
+              ~scope
 	  in
           f
-            ?fullsessname:(Option.map make_fullsessname state_name_oo)
-            ?cookie_scope:(Some cookie_type)
+            ?full_st_name:(Option.map make_full_st_name state_hier_oo)
+            ?cookie_level:(Some cookie_type)
             ~recompute_expdates:false
             true
             true
@@ -793,12 +795,22 @@ let parse_config hostpattern conf_info site_dir =
            | Some default_links_xhr ->
              sitedata.Eliom_common.default_links_xhr#set ~override_tenable:true default_links_xhr
            | None -> ());
+        Eliom_extension.register_eliom_extension 
+          default_module_action;
         (match parse_module_attrs None atts with
           | Some file_or_name ->
             exception_during_eliommodule_loading := true;
             load_eliom_module sitedata file_or_name content;
             exception_during_eliommodule_loading := false
           | _ -> ());
+
+      (*VVV 2012/08
+        It is not possible to load an eliom extension using <eliom>. Why?
+        Is there a reason for this? For now I fail in that case. *)
+        if Eliom_extension.get_eliom_extension () != default_module_action
+        then raise
+          (Error_in_config_file ("Eliom extensions cannot be loaded using <eliom>. Use <eliommodule> instead."));
+
         (* We must generate the page only if it is the first <eliom> tag 
            for that site: *)
         if !firsteliomtag
@@ -806,8 +818,12 @@ let parse_config hostpattern conf_info site_dir =
           firsteliomtag := false;
           Eliommod_pagegen.gen None sitedata
         end
-        else
+        else begin
+          if not !eliommodulewarningdisplayed
+          then Ocsigen_messages.warning "Tag <eliom> used several times in the same site: will run Eliom only the first time. Prefer <eliommodule> to load a module, and <eliom/> without attibute only once at the position you want to generate your Eliom pages for this site.";
+          eliommodulewarningdisplayed := true;
           gen_nothing ()
+        end
     | Element (t, _, _) ->
         raise (Ocsigen_extensions.Bad_config_tag_for_extension t)
     | _ -> raise (Error_in_config_file "(Eliommod extension)")
