@@ -1,14 +1,14 @@
-val close_data_session :
+val close_data_state :
   scope:[< Eliom_common.user_scope ] ->
   secure:bool option ->
   ?sp:Eliom_common.server_params -> unit -> unit
 val find_or_create_data_cookie :
   ?set_session_group:string ->
-  scope:[< Eliom_common.user_scope ] ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   secure:bool option ->
   ?sp:Eliom_common.server_params -> unit -> Eliom_common.one_data_cookie_info
 val find_data_cookie_only :
-  scope:[< Eliom_common.user_scope ] ->
+  cookie_scope:[< Eliom_common.cookie_scope ] ->
   secure:bool option ->
   ?sp:Eliom_common.server_params -> unit -> Eliom_common.one_data_cookie_info
 val counttableelements : (unit -> int) list ref
