@@ -20,21 +20,21 @@
 
 open Eliom_lib
 
-let make_full_named_group_name_ ~cookie_scope sitedata g =
-  (sitedata.Eliom_common.site_dir_string, cookie_scope, Left g)
+let make_full_named_group_name_ ~cookie_level sitedata g =
+  (sitedata.Eliom_common.site_dir_string, cookie_level, Left g)
 
-let make_full_group_name ~cookie_scope ri site_dir_string ipv4mask ipv6mask = 
+let make_full_group_name ~cookie_level ri site_dir_string ipv4mask ipv6mask = 
   function
   (* The scope is the scope of group members (`Session by default). *)
   | None -> (site_dir_string,
-             cookie_scope,
+             cookie_level,
              Right 
                (Ip_address.network_of_ip
                   (!!(ri.Ocsigen_extensions.ri_remote_ip_parsed))
                   ipv4mask
                   ipv6mask
                ))
-  | Some g -> (site_dir_string, cookie_scope, Left g)
+  | Some g -> (site_dir_string, cookie_level, Left g)
 
 let make_persistent_full_group_name =
   Eliom_common.make_persistent_full_group_name
@@ -47,33 +47,37 @@ module type MEMTAB =
     type group_of_group_data
 
     val add : ?set_max: int -> Eliom_common.sitedata ->
-      string -> Eliom_common.cookie_scope Eliom_common.sessgrp ->
+      string -> Eliom_common.cookie_level Eliom_common.sessgrp ->
       string Ocsigen_cache.Dlist.node
     val remove : 'a Ocsigen_cache.Dlist.node -> unit
-    val remove_group : Eliom_common.cookie_scope Eliom_common.sessgrp -> unit
+    val remove_group : Eliom_common.cookie_level Eliom_common.sessgrp -> unit
+
+    (** returns the dlist containing all session group elements *)
+    val find : [< Eliom_common.cookie_level ] Eliom_common.sessgrp ->
+      string Ocsigen_cache.Dlist.t
 
     (** Groups of browser sessions belongs to a group of groups.
         As these groups are not associated to a cookie,
         we put this information here. *)
     val find_node_in_group_of_groups : 
-      Eliom_common.cookie_scope Eliom_common.sessgrp ->
+      Eliom_common.cookie_level Eliom_common.sessgrp ->
       group_of_group_data option
 
     val move :
       ?set_max:int ->
       Eliom_common.sitedata ->
       string Ocsigen_cache.Dlist.node ->
-      Eliom_common.cookie_scope Eliom_common.sessgrp ->
+      Eliom_common.cookie_level Eliom_common.sessgrp ->
       string Ocsigen_cache.Dlist.node
 
     val up : string Ocsigen_cache.Dlist.node -> unit
     val nb_of_groups : unit -> int
-    val group_size : Eliom_common.cookie_scope Eliom_common.sessgrp -> int
+    val group_size : Eliom_common.cookie_level Eliom_common.sessgrp -> int
     val set_max : 'a Ocsigen_cache.Dlist.node -> int -> unit
   end
 
 module GroupTable = Hashtbl.Make(struct
-  type t = Eliom_common.cookie_scope Eliom_common.sessgrp
+  type t = Eliom_common.cookie_level Eliom_common.sessgrp
   let equal = (=)
   let hash = Hashtbl.hash
 end)
@@ -156,14 +160,14 @@ struct
         | None, _ -> assert false
         | Some v, _ -> v
       in
-      let cookie_scope = Tuple3.snd sess_grp in
+      let cookie_level = Tuple3.snd sess_grp in
       let cl = Ocsigen_cache.Dlist.create size in
       Ocsigen_cache.Dlist.set_finaliser_after
         (fun node ->
           let name = Ocsigen_cache.Dlist.value node in
           (* First we close all subsessions
              (that is, all sessions in the group associated to the session) *)
-          (match cookie_scope with
+          (match cookie_level with
 (*            | `Session_group -> assert false
               As there is no table of groups of groups
               (only one group of groups for each site),
@@ -171,7 +175,8 @@ struct
             | `Session (* We are closing a browser session *) ->
               (* First we close all tab sessions in the session (subgrp): *)
               let subgrp = 
-                make_full_named_group_name_ ~cookie_scope:`Client_process sitedata name 
+                make_full_named_group_name_
+                  ~cookie_level:`Client_process sitedata name 
               in
               remove_group subgrp
             | `Client_process (* We are closing a tab session *) -> ());
@@ -182,7 +187,7 @@ struct
           remove_if_empty sitedata sess_grp cl)
         cl;
       let node_in_group_of_group =
-        match cookie_scope with
+        match cookie_level with
           | `Session ->
             ignore (Ocsigen_cache.Dlist.add
                       sess_grp sitedata.Eliom_common.group_of_groups);
@@ -460,8 +465,8 @@ module Pers = struct
     | None -> Lwt.return []
       
 
-  let rec remove_group ~cookie_scope sitedata sess_grp =
-    (* cookie_scope is the scope of group members *)
+  let rec remove_group ~cookie_level sitedata sess_grp =
+    (* cookie_level is the scope of group members *)
 (*VVV NEW 201007 closing all sessions in the group and removing group data *)
 (*VVV VERIFY concurrent access *)
 (*VVV Check this carefully!!!! Verify the order of actions. *)
@@ -472,7 +477,7 @@ module Pers = struct
         find sess_grp >>= fun cl ->
         Lwt_util.iter
           (close_persistent_session2
-             ~cookie_scope:(match cookie_scope with
+             ~cookie_level:(match cookie_level with
                | `Client_process _ -> `Client_process | `Session -> `Session)
              sitedata None) cl
         (* None because we will close the group *)
@@ -493,9 +498,9 @@ module Pers = struct
         (* If it is associated to a session,
            we remove the session from its group,
            and we remove cookie info: *)
-        (match cookie_scope with
+        (match cookie_level with
           | `Client_process grp -> (* We are closing a browser session,
-                           belonging to the group grp *)
+                                      belonging to the group grp *)
             (* group_name is the cookie value *)
             remove sitedata group_name grp >>= fun () ->
             Ocsipersist.remove 
@@ -514,16 +519,15 @@ module Pers = struct
 
   (* close a persistent session (tab or browser) 
      and the associated group (if browser session) by cookie value *)
-  and close_persistent_session2 ~cookie_scope sitedata fullsessgrp cookie =
+  and close_persistent_session2 ~cookie_level sitedata fullsessgrp cookie =
 (*VVV Check this carefully!!!! *)
 (*VVV Optimize the number of marshal/unmarshal (getperssessgrp) *)
     Lwt.catch
       (fun () ->
-        match cookie_scope with
+        match cookie_level with
           | `Client_process -> begin
             (* We remove cookie info from the table *)
-            Ocsipersist.remove 
-              (!!Eliom_common.persistent_cookies_table) cookie
+            Ocsipersist.remove (!!Eliom_common.persistent_cookies_table) cookie
             >>= fun () ->
 
             (* We remove the session from its group: *)
@@ -534,10 +538,10 @@ module Pers = struct
           end
           | `Session ->
             remove_group
-              ~cookie_scope:(`Client_process fullsessgrp)
+              ~cookie_level:(`Client_process fullsessgrp)
               sitedata
               (Eliom_common.make_persistent_full_group_name
-                 ~cookie_scope:`Client_process
+                 ~cookie_level:`Client_process
                  sitedata.Eliom_common.site_dir_string
                  (Some cookie))
       )
@@ -564,7 +568,7 @@ module Pers = struct
               *)
               (match Eliom_common.getperssessgrp sg0 with
                 | (_, `Session, _) ->
-                  remove_group ~cookie_scope:`Session sitedata sess_grp
+                  remove_group ~cookie_level:`Session sitedata sess_grp
                 | _ -> Lwt.return ()
               ) >>= fun () ->
               Ocsipersist.remove !!grouptable sg
