@@ -1,25 +1,22 @@
 
-(** TOOO dump intermediate file *)
-
 open Utils
 
 let usage () =
   Printf.eprintf "Usage: %s <options> <files>\n" (Filename.basename Sys.argv.(0));
   Printf.eprintf "SPECIFIC OPTIONS:\n";
+  Printf.eprintf "  -package <name>\tRefer to package when compiling\n";
+  if !kind <> `Client then
+    Printf.eprintf "  -infer\t\tOnly infer the type of values sent by the server\n";
   Printf.eprintf "  -dir <dir>\t\tThe directory for generated files (default %S)\n"
     (if !kind = `Client then default_client_dir else default_server_dir);
-  if !kind =  `Server || !kind = `ServerOpt then begin
-    Printf.eprintf "  -infer\t\tOnly infer the type of values sent by the server\n";
-    Printf.eprintf "  -noinfer\t\tDo not infer the type of values sent by the server\n";
-  end else begin
+  Printf.eprintf "  -type-dir <dir>\tThe directory to read server type files from (default %S)\n"
+     default_type_dir;
+  Printf.eprintf "  -server-types-ext <ext> The extension for server type files (default %S)\n"
+    default_server_types_ext;
+  if !kind = `Client then
     Printf.eprintf "  -jsopt <opt>\t\tAppend option <opt> to js_of_ocaml invocation\n";
-  end;
-  Printf.eprintf "  -package <name>\tRefer to package when compiling\n";
+  Printf.eprintf "  -ppopt <p>\t\tAppend option <opt> to preprocessor invocation\n";
   Printf.eprintf "  -predicates <p>\tAdd predicate <p> when resolving package properties\n";
-  Printf.eprintf "  -ppopt <p>\tAppend option <opt> to preprocessor invocation\n";
-  if !kind = `Client then begin
-    Printf.eprintf "  -type <file>\tInfered types for the values sent by the server.\n";
-  end;
   create_filter !compiler ["-help"] (help_filter 2 "STANDARD OPTIONS:");
   if !kind = `Client then
     create_filter !js_of_ocaml ["-help"] (help_filter 1 "JS_OF_OCAML OPTIONS:");
@@ -29,14 +26,14 @@ let usage () =
 
 let jsopt : string list ref = ref []
 let output_name : string option ref = ref None
-let noinfer = ref false
 
-let mode : [ `Link | `Compile | `InferOnly | `Library  | `Pack | `Obj | `Shared ] ref =
-  ref `Link
+type mode = [ `Link | `Compile | `Infer | `Library  | `Pack | `Obj | `Shared | `Interface ]
 
-let do_compile () = !mode <> `InferOnly
-let do_infer () = not !noinfer
-(* let do_dump = ref false *)
+let mode : mode ref = ref `Link
+
+let do_compile () = !mode <> `Infer
+let do_interface () = !mode = `Interface
+let do_dump = ref false
 
 let create_process ?in_ ?out ?err name args =
   wait (create_process ?in_ ?out ?err name args)
@@ -59,31 +56,37 @@ let output_prefix ?(ty = false) name =
   let name =
     match !output_name with
     | None ->
-	if !mode = `InferOnly || ty
+	if !mode = `Infer || ty
 	then prefix_type_dir name
 	else prefix_output_dir name
     | Some n ->
-	if !mode = `Compile || !mode = `InferOnly
+	if !mode = `Compile || !mode = `Infer
 	then (output_name := None; n)
 	else prefix_output_dir name in
   check_or_create_dir (Filename.dirname name);
   chop_extension_if_any name
 
 let set_mode m =
- if !mode =  `Link then begin
-   if m = `Shared && !kind <> `ServerOpt then usage ();
-   if m = `InferOnly && (!kind <> `Server && !kind <> `ServerOpt) then usage ();
-   mode := m
- end else
-    match !kind with
-      | `ServerOpt ->
-	  Printf.eprintf
-	    "Please specify at most one of -pack, -a, -shared, -c, -infer, -output-obj\n%!";
-	exit 1
-      | `Client | `Server ->
-	  Printf.eprintf
-	    "Please specify at most one of -pack, -a, -c, -infer, -output-obj\n%!";
-	exit 1
+ if !mode = `Link then
+   ( if
+       (m = `Shared && !kind <> `ServerOpt) ||
+       (m = `Infer && !kind = `Client) ||
+       (m = `Interface && !kind = `Client)
+     then
+       usage ()
+     else
+       mode := m )
+ else
+   let args =
+     let basic_args = ["-pack"; "-a"; "-c"; "output-obj"] in
+     let infer_args = if !kind <> `Client then ["-i"; "-infer"] else [] in
+     let shared_args = if !kind <> `ServerOpt then ["-shared"] else [] in
+     basic_args @ infer_args @ shared_args
+   in
+   Printf.eprintf
+     "Please specify at most one of %s\n%!"
+     (String.concat ", " args);
+  exit 1
 
 let get_product_name () = match !output_name with
   | None ->
@@ -120,70 +123,126 @@ let get_thread_opt () = match !kind with
 
 let obj_ext () = if !kind = `ServerOpt then ".cmx" else ".cmo"
 
-let compile_intf file =
-  if do_compile () then
-    let obj = output_prefix file ^ ".cmi" in
-    create_process !compiler ( ["-c" ; "-o" ; obj ] @ get_pp [] @ !args
-			       @ get_thread_opt ()
-			       @ get_common_include ()
-			       @ ["-intf"; file] )
+(* Process ml and mli files *)
 
-let compile_impl file =
+let compile_ocaml ~impl_intf file =
+  let obj =
+    let ext = match impl_intf with
+      | `Impl -> obj_ext ()
+      | `Intf -> ".cmi"
+    in
+    output_prefix file ^ ext in
+  create_process !compiler ( ["-c" ;
+                              "-o" ; obj ;
+                              "-pp"; get_pp []]
+                             @ !args
+			     @ get_thread_opt ()
+			     @ get_common_include ()
+			     @ [impl_intf_opt impl_intf; file] )
+
+let output_ocaml_interface file =
+    create_process !compiler ( ["-i"; "-pp"; get_pp []] @ !args
+                               @ get_common_include ()
+                               @ [file] )
+
+let process_ocaml ~impl_intf file =
   if do_compile () then
-    let obj = output_prefix file ^ obj_ext () in
-    create_process !compiler ( ["-c" ; "-o"  ; obj ] @ get_pp [] @ !args
-			       @ get_thread_opt ()
-			       @ get_common_include ()
-			       @ ["-impl"; file] );
-    args := !args @ [obj]
+    compile_ocaml ~impl_intf file;
+  if !mode = `Interface then
+    output_ocaml_interface file
 
 let compile_obj file =
   if do_compile () then
     ( create_process !compiler (!args @ [file]);
       args := !args @ [output_prefix file ^ ext_obj] )
 
+(* Process eliom and eliomi files *)
+
+let get_ppopts ~impl_intf file =
+  let pa_cmo =
+    match !kind with
+      | `Client -> "pa_eliom_client_client.cmo"
+      | `Server | `ServerOpt -> "pa_eliom_client_server.cmo"
+  in
+  pa_cmo :: type_opt impl_intf file @ !ppopt @ [impl_intf_opt impl_intf]
+
 let compile_server_type_eliom file =
-  if do_infer () then
-    let obj = output_prefix ~ty:true file ^ type_file_suffix in
-    let ppopt = ["pa_eliom_type_filter.cmo"; "-impl"] in
-    let out = Unix.openfile obj [Unix.O_WRONLY; Unix.O_CREAT; Unix.O_TRUNC] 0o666 in
-    create_process ~out !compiler ( [ "-i" ; "-thread" ]
-				    @ get_pp ppopt
-				    @ !args
-				    @ get_common_include ()
-				    @ ["-impl"; file] );
-    Unix.close out
+  let obj = output_prefix ~ty:true file ^ !server_types_file_ext
+  and ppopts = ["pa_eliom_type_filter.cmo"] @ !ppopt @ ["-impl"] in
+  if !do_dump then begin
+    let camlp4, ppopt = get_pp_dump ("-printer" :: "o" :: ppopts @ [file]) in
+    create_process camlp4 ppopt;
+    exit 0
+  end;
+  let out = Unix.openfile obj [Unix.O_WRONLY; Unix.O_CREAT; Unix.O_TRUNC] 0o666 in
+  create_process ~out !compiler ( [ "-i" ; "-pp"; get_pp ppopts]
+				  @ !args
+    				  @ get_common_include ()
+				  @ ["-impl"; file] );
+  Unix.close out
 
-let compile_server_eliom file =
-  if do_compile () then
-    let obj = output_prefix file ^ obj_ext () in
-    let ppopt = ["pa_eliom_client_server.cmo"; "-impl"] in
-    create_process !compiler ( [ "-c" ; "-thread" ; "-o"  ; obj ]
-			       @ get_pp ppopt
-			       @ !args
-			       @ get_common_include ()
-			       @ ["-impl"; file] );
-    args := !args @ [obj]
+let output_eliom_interface ~impl_intf file =
+  let ppopts = get_ppopts ~impl_intf file in
+  let indent ch =
+    try
+      while true do
+        let line = input_line ch in
+        Printf.printf "  %s\n" line
+      done
+    with End_of_file -> ()
+  in
+  Printf.printf "(* WARNING `eliomc -i' generated this pretty ad-hoc - use with care! *)\n";
+  Printf.printf "{server{\n";
+    create_filter !compiler ( [ "-i" ; "-pp" ; get_pp ppopts; "-intf-suffix"; ".eliomi" ]
+			         @ !args
+			         @ get_common_include ~kind:`Server ()
+			         @ [impl_intf_opt impl_intf; file] )
+      indent;
+  Printf.printf "}}\n";
+  Printf.printf "{client{\n";
+    create_filter !compiler ( [ "-i" ; "-pp" ; get_pp ppopts; "-intf-suffix"; ".eliomi" ]
+			         @ !args
+			         @ get_common_include ~kind:`Server ()
+			         @ [impl_intf_opt impl_intf; file] )
+      indent;
+  Printf.printf "}}\n"
 
-let compile_client_eliom file =
-  let obj = output_prefix file ^ ".cmo" in
-  let ppopt = ["pa_eliom_client_client.cmo"; "-type" ; get_type_file file; "-impl"] in
-  create_process !compiler ( ["-c" ; "-o"  ; obj ]
-			     @ get_pp ppopt
+let compile_eliom ~impl_intf file =
+  let obj =
+    let ext = match !kind with
+      | `Client -> ".cmo"
+      | `Server | `ServerOpt -> obj_ext ()
+    in
+    output_prefix file ^ ext
+  in
+  let ppopts = get_ppopts ~impl_intf file in
+  if !do_dump then begin
+    let camlp4, ppopt = get_pp_dump ("-printer" :: "o" :: ppopts @ [file]) in
+    create_process camlp4 ppopt;
+    exit 0
+  end;
+  create_process !compiler ( [ "-c" ;
+                               "-o"  ; obj ;
+                               "-pp" ; get_pp ppopts;
+                               "-intf-suffix"; ".eliomi" ]
+                             @ get_thread_opt ()
 			     @ !args
 			     @ get_common_include ()
-			     @ ["-impl"; file] );
+			     @ [impl_intf_opt impl_intf; file] );
   args := !args @ [obj]
 
-let compile_eliom file = match !kind with
-  | `Client ->
-    compile_client_eliom file
-  | `Server | `ServerOpt ->
-    compile_server_eliom file;
-    compile_server_type_eliom file
+let process_eliom ~impl_intf file =
+  match !mode with
+    | `Infer when impl_intf = `Impl ->
+      compile_server_type_eliom file
+    | `Interface ->
+      output_eliom_interface ~impl_intf file
+    | _ ->
+      compile_eliom ~impl_intf file
 
-let build_server ?(name = "a.out") () = ()
-    (* TODO ? Build a staticaly linked ocsigenserver. *)
+let build_server ?(name = "a.out") () =
+  fail "Linking eliom server is not yet supported"
+  (* TODO ? Build a staticaly linked ocsigenserver. *)
 
 let build_client () =
   let name = chop_extension_if_any (get_product_name ()) in
@@ -204,21 +263,22 @@ let rec process_option () =
   while !i < Array.length Sys.argv do
     match Sys.argv.(!i) with
     | "-help" | "--help" -> usage ()
-    | "-i" -> todo ()
+    | "-i" -> set_mode `Interface; incr i
     | "-c" -> set_mode `Compile; incr i
     | "-a" -> set_mode `Library; incr i
     | "-pack" -> set_mode `Pack; incr i
     | "-output-obj" -> set_mode `Obj; incr i
     | "-shared" -> set_mode `Shared; incr i
-    | "-infer" -> set_mode `InferOnly; incr i
+    | "-infer" -> set_mode `Infer; incr i
     | "-verbose" -> verbose := true; args := !args @ ["-verbose"] ;incr i
-    | "-noinfer" when !kind = `Server || !kind = `ServerOpt ->
-	noinfer := true; incr i
     | "-vmthread" -> Printf.eprintf "The -vmthread option isn't supported yet."; exit 1
     | "-o" ->
       if !i+1 >= Array.length Sys.argv then usage ();
       output_name := Some Sys.argv.(!i+1);
       i := !i+2
+    | "-dump" ->
+      do_dump := not !do_dump;
+      i := !i+1
     | "-dir" ->
       if !i+1 >= Array.length Sys.argv then usage ();
       build_dir := Sys.argv.(!i+1);
@@ -227,6 +287,10 @@ let rec process_option () =
       if !i+1 >= Array.length Sys.argv then usage ();
       type_dir := Sys.argv.(!i+1);
       i := !i+2
+    | "-server-types-ext" ->
+      if !i+1 >= Array.length Sys.argv then usage ();
+      server_types_file_ext := Sys.argv.(!i+1);
+      i := !i+2
     | "-package" ->
       if !i+1 >= Array.length Sys.argv then usage ();
       package := !package @ split ',' Sys.argv.(!i+1);
@@ -248,25 +312,25 @@ let rec process_option () =
       if !i+1 >= Array.length Sys.argv then usage ();
       ppopt := !ppopt @ [Sys.argv.(!i+1)];
       i := !i+2
-    | "-type" ->
-      if !kind <> `Client then usage ();
-      if !i+1 >= Array.length Sys.argv then usage ();
-      type_file := Some Sys.argv.(!i+1);
-      i := !i+2
     | "-intf" ->
       if !i+1 >= Array.length Sys.argv then usage ();
-      compile_intf Sys.argv.(!i+1);
+      process_eliom ~impl_intf:`Intf Sys.argv.(!i+1);
       i := !i+2
     | "-impl" ->
       if !i+1 >= Array.length Sys.argv then usage ();
-      compile_eliom Sys.argv.(!i+1);
+      process_eliom ~impl_intf:`Impl Sys.argv.(!i+1);
       i := !i+2
     | arg when Filename.check_suffix arg ".mli" ->
-      compile_intf arg; incr i
+      process_ocaml ~impl_intf:`Intf arg;
+      incr i
     | arg when Filename.check_suffix arg ".ml" ->
-      compile_impl arg; incr i
+      process_ocaml ~impl_intf:`Impl arg;
+      incr i
     | arg when Filename.check_suffix arg ".eliom" ->
-      compile_eliom arg;
+      process_eliom ~impl_intf:`Impl arg;
+      incr i
+    | arg when Filename.check_suffix arg ".eliomi" ->
+      process_eliom ~impl_intf:`Intf arg;
       incr i
     | arg when Filename.check_suffix arg ".c" ->
       compile_obj arg; incr i
@@ -279,7 +343,7 @@ let rec process_option () =
   | `Shared -> build_shared ()
   | `Link when !kind = `Client -> build_client ()
   | `Link (* Server and ServerOpt *) -> build_server ?name:(!output_name) ()
-  | `Compile | `InferOnly -> ()
+  | `Compile | `Infer | `Interface -> ()
 
 let main () =
   let k =
