@@ -56,9 +56,18 @@ let default_type_dir =
   try Sys.getenv "ELIOM_TYPE_DIR"
   with Not_found -> default_server_dir
 
+let default_server_types_ext =
+  try Sys.getenv "ELIOM_SERVER_TYPES_EXT"
+  with Not_found -> ".type_mli"
+
 let build_dir : string ref = ref ""
 let type_dir : string ref = ref default_type_dir
 
+let get_kind ~k =
+  match k with
+    | Some k -> k
+    | None -> !kind
+
 (** Findlib *)
 
 let _ = Findlib.init ()
@@ -72,25 +81,31 @@ let ocamldep = Findlib.command `ocamldep
 let js_of_ocaml = ref "js_of_ocaml"
 
 let compiler = ref ocamlc
+let camlp4 = ref "camlp4o"
 
 let ppopt : string list ref = ref []
 
-let get_predicates () = match !kind with
+let get_predicates ?kind:k () = match get_kind k with
   | `Server -> ["mt"; "byte"] @ !predicates
   | `Client -> ["byte"] @ !predicates
   | `ServerOpt -> ["native"] @ !predicates
 
 let syntax_predicates = ["preprocessor";"syntax";"camlp4o"] @ !predicates
 
-let get_server_package () =
+let get_server_package ?kind:k ?package:p () =
+  let package =
+    match p with
+      | Some p -> p
+      | None -> !package
+  in
   try
-    Findlib.package_deep_ancestors (get_predicates ()) ("eliom.server" :: !package)
+    Findlib.package_deep_ancestors (get_predicates ?kind:k ()) ("eliom.server" :: package)
   with Findlib.No_such_package (name, _) ->
     Printf.eprintf "Unknown package: %s\n%!" name;
     exit 1
-let get_client_package () =
+let get_client_package ?kind:k () =
   try
-    Findlib.package_deep_ancestors (get_predicates ()) ("eliom.client" :: !package)
+    Findlib.package_deep_ancestors (get_predicates ?kind:k ()) ("eliom.client" :: !package)
   with Findlib.No_such_package (name, _) ->
     Printf.eprintf "Unknown package: %s\n%!" name;
     exit 1
@@ -105,13 +120,14 @@ let rec map_include xs = match xs with
   | [] -> []
   | x::xs -> "-I" :: x :: map_include xs
 
-let get_common_include () =
-  (match !kind with
+let get_common_include ?kind:k ?build_dir:dir ?package:p () =
+  let dir = match dir with Some d -> d | None -> !build_dir in
+  (match get_kind ~k with
   | `Server | `ServerOpt ->
-      map_include (List.map Findlib.package_directory (get_server_package ()))
+      map_include (List.map Findlib.package_directory (get_server_package ?kind:k ?package:p ()))
   | `Client ->
-      map_include (List.map Findlib.package_directory (get_client_package ())))
-  @ match !build_dir with
+      map_include (List.map Findlib.package_directory (get_client_package ?kind:k ())))
+  @ match dir with
     | "" | "." -> []
     | d -> ["-I"; d]
 
@@ -127,14 +143,14 @@ let get_common_syntax () =
 	 with Not_found -> [])
        (get_syntax_package ()))
 
-let get_client_lib () =
+let get_client_lib ?kind:k () =
   List.concat
     (List.map
        (fun p ->
 	 try
-	   split ' ' (Findlib.package_property (get_predicates ()) p "archive")
+	   split ' ' (Findlib.package_property (get_predicates ?kind:k ()) p "archive")
 	 with Not_found -> [])
-       (get_client_package ()))
+       (get_client_package ?kind:k ()))
 
 let get_client_js () =
   try
@@ -143,15 +159,25 @@ let get_client_js () =
     Printf.eprintf "Unknown package: %s\n%!" name;
     exit 1
 
+(* Should be calld only with -dump... *)
+let get_pp_dump opt = match !pp with
+  | None -> (!camlp4, get_common_syntax () @ opt)
+  | Some pp ->
+      try
+        ignore(String.index pp ' ');
+        Printf.eprintf "Incompatible option: -pp and -dump\n%!";
+        exit 1
+      with Not_found -> (pp, get_common_syntax () @ opt)
+
 let get_pp opt = match !pp with
-  | None -> ["-pp"; String.concat " " ("camlp4" :: !ppopt @ get_common_syntax () @ opt)]
-  | Some pp -> ["-pp"; pp ^ " " ^ String.concat " " (!ppopt @ get_common_syntax () @ opt)]
+  | None -> String.concat " " (!camlp4 :: get_common_syntax () @ opt)
+  | Some pp -> pp ^ " " ^ String.concat " " (get_common_syntax () @ opt)
 
 let get_thread_opt () = match !kind with
   | `Client -> []
   | `Server | `ServerOpt -> ["-thread"]
 
-let type_file_suffix = ".type_mli"
+let server_types_file_ext = ref default_server_types_ext
 
 let prefix_type_dir name =
   match !type_dir with
@@ -162,7 +188,16 @@ let get_type_file file =
   match !type_file with
   | Some f -> f
   | None ->
-      prefix_type_dir (chop_extension_if_any file ^ type_file_suffix)
+      prefix_type_dir (chop_extension_if_any file ^ !server_types_file_ext)
+
+let impl_intf_opt = function
+  | `Impl -> "-impl"
+  | `Intf -> "-intf"
+
+let type_opt impl_intf file =
+  match impl_intf with
+  | `Impl -> ["-type"; get_type_file file]
+  | `Intf -> ["-notype"]
 
 (** Process *)
 
@@ -199,6 +234,13 @@ let help_filter skip msg ch =
   prerr_endline msg;
   while true do prerr_endline (input_line ch) done
 
+let fail fmt =
+  Printf.ksprintf
+    (fun msg ->
+      prerr_endline msg;
+      exit 1)
+    fmt
+
 (** *)
 
 (** *)
