@@ -1,29 +1,32 @@
-open Eliom_compatibility_2_1
+
 let (>>=) = Lwt.(>>=)
 let (>|=) = Lwt.(>|=)
 
-open HTML5.M
-open Eliom_output.Html5
+open Eliom_lib
+open Eliom_content
+open Html5.D
+
 open Eliom_testsuite1
 open Eliom_testsuite2
 open Eliom_testsuite3
 
 (* Main page for the test suite *)
-let _ = Eliom_output.Html5.register Eliom_testsuite3.main
+let _ = Eliom_registration.Html5.register Eliom_testsuite_base.main
   (fun () () ->
     Lwt.return
      (html
        (head
           (title (pcdata "Examples from the manual"))
-          [Eliom_output.Html5.css_link 
-              (Eliom_output.Html5.make_uri
+          [Html5.D.css_link
+              (Html5.D.make_uri
                  ~service:(Eliom_service.static_dir ()) ["style.css"]) ()])
        (body
           [
             h1 [img ~alt:"Ocsigen"
-                   ~src:(Eliom_output.Html5.make_uri
+                   ~src:(Html5.D.make_uri
                            ~service:(Eliom_service.static_dir ()) ["ocsigen5.png"]) ()];
 
+            Eliom_testsuite_base.testsuite ~name:"Test suite 4" Eliom_testsuite4.tests;
             h3 [pcdata "Eliom examples"];
             h4 [pcdata "Simple pages"];
             p [
@@ -45,6 +48,11 @@ let _ = Eliom_output.Html5.register Eliom_testsuite3.main
               a default [code [pcdata "rep/"]] ()
             ];
 
+            h4 [pcdata "Destillery"];
+            p [
+              Raw.a ~a:[a_href (Raw.uri_of_string "destillery/basic")] [pcdata "Basic destillery"];
+            ];
+
             h4 [pcdata "Parameters"];
             p [
               pcdata "A page with GET parameters: ";
@@ -86,7 +94,7 @@ let _ = Eliom_output.Html5.register Eliom_testsuite3.main
 
 
               pcdata "The ";
-              a main [pcdata "default page"] ();
+              a Eliom_testsuite_base.main [pcdata "default page"] ();
               pcdata "of this directory (myself)";
               br ();
 
@@ -253,6 +261,7 @@ let _ = Eliom_output.Html5.register Eliom_testsuite3.main
 
               a Eliom_testsuite_site.reference_scope_site [pcdata "References of scope site"] ();
               br ();
+
             ];
 
             h4 [pcdata "Advanced forms"];
@@ -289,7 +298,8 @@ let _ = Eliom_output.Html5.register Eliom_testsuite3.main
             h3 [pcdata "js_of_ocaml events"];
 
             p [
-              a event_service [code [pcdata "Test suite"]] ();
+              a event_service [code [pcdata "With arrows"]] (); br ();
+              a event2_service [code [pcdata "With Lwt"]] ();
               br ();
             ];
 
@@ -300,6 +310,7 @@ let _ = Eliom_output.Html5.register Eliom_testsuite3.main
          a sumform [pcdata "alternative parameters"] (); br ();
          a sumform2 [pcdata "alternative parameters with POST"] (); br ();
          a optform [pcdata "Optional parameters"] (); br ();
+         a main_neopt_service [pcdata "Non-empty optional parameters"] (); br ();
          a sfail [pcdata "Service raising an exception"] (); br ();
          a sraise [pcdata "Wrong use of exceptions during service"] (); br ();
          a getcoex [pcdata "GET coservice with preapplied fallback, etc"] (); br ();
@@ -484,6 +495,8 @@ let _ = Eliom_output.Html5.register Eliom_testsuite3.main
 
             h4 [pcdata "Process states"];
             p [
+              a states_test [pcdata "Extensive test of Eliom references of different scopes, accessed from inside or outside the state itself"] ();
+              br ();
               pcdata "Coservices: ";
               a tcoservices_example [code [pcdata "tcoservice"]] ();
               br ();
@@ -547,4 +560,3 @@ let _ = Eliom_output.Html5.register Eliom_testsuite3.main
           ])))
 
 (* *zap*)
-
